(function (global, factory) {
  typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports, require('@stoplight/path')) :
  typeof define === 'function' && define.amd ? define(['exports', '@stoplight/path'], factory) :
  (factory((global.jsonRefResolver = {}),global.path));
}(this, (function (exports,path) { 'use strict';

  path = path && path.hasOwnProperty('default') ? path['default'] : path;

  var Cache = /** @class */ (function () {
      function Cache(opts) {
          if (opts === void 0) { opts = {}; }
          this.debug = false;
          this._stats = {
              hits: 0,
              misses: 0,
          };
          // private readonly _errTTL = 0;
          this._data = {};
          this._stdTTL = opts.stdTTL;
      }
      Object.defineProperty(Cache.prototype, "stats", {
          get: function () {
              return this._stats;
          },
          enumerable: true,
          configurable: true
      });
      Cache.prototype.get = function (key) {
          var d = this._data[key];
          // if we have a value and it is fresh, use it
          if (d && (!this._stdTTL || new Date().getTime() - d.ts < this._stdTTL)) {
              this._stats.hits += 1;
              if (this.debug) {
                  console.log('cache.get.hit', { key: key, val: d });
              }
              return d.val;
          }
          this._stats.misses += 1;
          if (this.debug) {
              console.log('cache.get.miss', { key: key });
          }
      };
      Cache.prototype.set = function (key, val) {
          if (this.debug) {
              console.log('cache.set', { key: key, val: val });
          }
          this._data[key] = {
              ts: new Date().getTime(),
              val: val,
          };
      };
      Cache.prototype.has = function (key) {
          return key in this._data;
      };
      return Cache;
  }());

  /*! *****************************************************************************
  Copyright (c) Microsoft Corporation. All rights reserved.
  Licensed under the Apache License, Version 2.0 (the "License"); you may not use
  this file except in compliance with the License. You may obtain a copy of the
  License at http://www.apache.org/licenses/LICENSE-2.0

  THIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
  KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED
  WARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE,
  MERCHANTABLITY OR NON-INFRINGEMENT.

  See the Apache Version 2.0 License for specific language governing permissions
  and limitations under the License.
  ***************************************************************************** */

  var __assign = function() {
      __assign = Object.assign || function __assign(t) {
          for (var s, i = 1, n = arguments.length; i < n; i++) {
              s = arguments[i];
              for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p)) t[p] = s[p];
          }
          return t;
      };
      return __assign.apply(this, arguments);
  };

  function __awaiter(thisArg, _arguments, P, generator) {
      return new (P || (P = Promise))(function (resolve, reject) {
          function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
          function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
          function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
          step((generator = generator.apply(thisArg, _arguments || [])).next());
      });
  }

  function __generator(thisArg, body) {
      var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
      return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
      function verb(n) { return function (v) { return step([n, v]); }; }
      function step(op) {
          if (f) throw new TypeError("Generator is already executing.");
          while (_) try {
              if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
              if (y = 0, t) op = [op[0] & 2, t.value];
              switch (op[0]) {
                  case 0: case 1: t = op; break;
                  case 4: _.label++; return { value: op[1], done: false };
                  case 5: _.label++; y = op[1]; op = [0]; continue;
                  case 7: op = _.ops.pop(); _.trys.pop(); continue;
                  default:
                      if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                      if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                      if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                      if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                      if (t[2]) _.ops.pop();
                      _.trys.pop(); continue;
              }
              op = body.call(thisArg, _);
          } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
          if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
      }
  }

  var commonjsGlobal = typeof window !== 'undefined' ? window : typeof global !== 'undefined' ? global : typeof self !== 'undefined' ? self : {};

  function unwrapExports (x) {
  	return x && x.__esModule && Object.prototype.hasOwnProperty.call(x, 'default') ? x.default : x;
  }

  function createCommonjsModule(fn, module) {
  	return module = { exports: {} }, fn(module, module.exports), module.exports;
  }

  var fastSafeStringify = stringify;
  stringify.default = stringify;
  stringify.stable = deterministicStringify;
  stringify.stableStringify = deterministicStringify;
  stringify.decycle = decycle;

  const arr = [];

  // Regular stringify
  function stringify (obj, replacer, spacer) {
    decirc(obj, '', [], undefined);
    const res = JSON.stringify(obj, replacer, spacer);
    while (arr.length !== 0) {
      var part = arr.pop();
      part[0][part[1]] = part[2];
    }
    return res
  }

  function decycle (val, replacer) {
    decirc(val, '', [], undefined, replacer);
    return val
  }

  function defaultReplacer (val, parentKey, stack, parent) {
    arr.push([parent, parentKey, val]);
    return '[Circular]'
  }

  function decirc (val, parentKey, stack, parent, replacer) {
    replacer = replacer || defaultReplacer;

    var i;
    if (typeof val === 'object' && val !== null) {
      for (i = 0; i < stack.length; i++) {
        if (stack[i][0] === val) {
          parent[parentKey] = replacer(val, parentKey, stack, parent);
          return
        }
      }
      stack.push([val, parentKey]);
      // Optimize for Arrays. Big arrays could kill the performance otherwise!
      if (Array.isArray(val)) {
        for (i = 0; i < val.length; i++) {
          decirc(val[i], i, stack, val, replacer);
        }
      } else {
        const keys = Object.keys(val);
        for (i = 0; i < keys.length; i++) {
          var key = keys[i];
          decirc(val[key], key, stack, val, replacer);
        }
      }
      stack.pop();
    }
  }

  // Stable-stringify
  function compareFunction (a, b) {
    if (a < b) {
      return -1
    }
    if (a > b) {
      return 1
    }
    return 0
  }

  function deterministicStringify (obj, replacer, spacer) {
    const tmp = deterministicDecirc(obj, '', [], undefined) || obj;
    const res = JSON.stringify(tmp, replacer, spacer);
    while (arr.length !== 0) {
      var part = arr.pop();
      part[0][part[1]] = part[2];
    }
    return res
  }

  function deterministicDecirc (val, k, stack, parent) {
    var i;
    if (typeof val === 'object' && val !== null) {
      for (i = 0; i < stack.length; i++) {
        if (stack[i] === val) {
          parent[k] = '[Circular]';
          arr.push([parent, k, val]);
          return
        }
      }
      if (typeof val.toJSON === 'function') {
        return
      }
      stack.push(val);
      // Optimize for Arrays. Big arrays could kill the performance otherwise!
      if (Array.isArray(val)) {
        for (i = 0; i < val.length; i++) {
          deterministicDecirc(val[i], i, stack, val);
        }
      } else {
        // Create a temporary object in the required way
        const tmp = {};
        const keys = Object.keys(val).sort(compareFunction);
        for (i = 0; i < keys.length; i++) {
          var key = keys[i];
          deterministicDecirc(val[key], key, stack, val);
          tmp[key] = val[key];
        }
        if (parent !== undefined) {
          arr.push([parent, k, val]);
          parent[k] = tmp;
        } else {
          return tmp
        }
      }
      stack.pop();
    }
  }

  /**
   * lodash (Custom Build) <https://lodash.com/>
   * Build: `lodash modularize exports="npm" -o ./`
   * Copyright jQuery Foundation and other contributors <https://jquery.org/>
   * Released under MIT license <https://lodash.com/license>
   * Based on Underscore.js 1.8.3 <http://underscorejs.org/LICENSE>
   * Copyright Jeremy Ashkenas, DocumentCloud and Investigative Reporters & Editors
   */

  /** Used as the `TypeError` message for "Functions" methods. */
  var FUNC_ERROR_TEXT = 'Expected a function';

  /** Used to stand-in for `undefined` hash values. */
  var HASH_UNDEFINED = '__lodash_hash_undefined__';

  /** Used as references for various `Number` constants. */
  var INFINITY = 1 / 0;

  /** `Object#toString` result references. */
  var funcTag = '[object Function]',
      genTag = '[object GeneratorFunction]',
      symbolTag = '[object Symbol]';

  /** Used to match property names within property paths. */
  var reIsDeepProp = /\.|\[(?:[^[\]]*|(["'])(?:(?!\1)[^\\]|\\.)*?\1)\]/,
      reIsPlainProp = /^\w*$/,
      reLeadingDot = /^\./,
      rePropName = /[^.[\]]+|\[(?:(-?\d+(?:\.\d+)?)|(["'])((?:(?!\2)[^\\]|\\.)*?)\2)\]|(?=(?:\.|\[\])(?:\.|\[\]|$))/g;

  /**
   * Used to match `RegExp`
   * [syntax characters](http://ecma-international.org/ecma-262/7.0/#sec-patterns).
   */
  var reRegExpChar = /[\\^$.*+?()[\]{}|]/g;

  /** Used to match backslashes in property paths. */
  var reEscapeChar = /\\(\\)?/g;

  /** Used to detect host constructors (Safari). */
  var reIsHostCtor = /^\[object .+?Constructor\]$/;

  /** Detect free variable `global` from Node.js. */
  var freeGlobal = typeof commonjsGlobal == 'object' && commonjsGlobal && commonjsGlobal.Object === Object && commonjsGlobal;

  /** Detect free variable `self`. */
  var freeSelf = typeof self == 'object' && self && self.Object === Object && self;

  /** Used as a reference to the global object. */
  var root = freeGlobal || freeSelf || Function('return this')();

  /**
   * Gets the value at `key` of `object`.
   *
   * @private
   * @param {Object} [object] The object to query.
   * @param {string} key The key of the property to get.
   * @returns {*} Returns the property value.
   */
  function getValue(object, key) {
    return object == null ? undefined : object[key];
  }

  /**
   * Checks if `value` is a host object in IE < 9.
   *
   * @private
   * @param {*} value The value to check.
   * @returns {boolean} Returns `true` if `value` is a host object, else `false`.
   */
  function isHostObject(value) {
    // Many host objects are `Object` objects that can coerce to strings
    // despite having improperly defined `toString` methods.
    var result = false;
    if (value != null && typeof value.toString != 'function') {
      try {
        result = !!(value + '');
      } catch (e) {}
    }
    return result;
  }

  /** Used for built-in method references. */
  var arrayProto = Array.prototype,
      funcProto = Function.prototype,
      objectProto = Object.prototype;

  /** Used to detect overreaching core-js shims. */
  var coreJsData = root['__core-js_shared__'];

  /** Used to detect methods masquerading as native. */
  var maskSrcKey = (function() {
    var uid = /[^.]+$/.exec(coreJsData && coreJsData.keys && coreJsData.keys.IE_PROTO || '');
    return uid ? ('Symbol(src)_1.' + uid) : '';
  }());

  /** Used to resolve the decompiled source of functions. */
  var funcToString = funcProto.toString;

  /** Used to check objects for own properties. */
  var hasOwnProperty$1 = objectProto.hasOwnProperty;

  /**
   * Used to resolve the
   * [`toStringTag`](http://ecma-international.org/ecma-262/7.0/#sec-object.prototype.tostring)
   * of values.
   */
  var objectToString = objectProto.toString;

  /** Used to detect if a method is native. */
  var reIsNative = RegExp('^' +
    funcToString.call(hasOwnProperty$1).replace(reRegExpChar, '\\$&')
    .replace(/hasOwnProperty|(function).*?(?=\\\()| for .+?(?=\\\])/g, '$1.*?') + '$'
  );

  /** Built-in value references. */
  var Symbol$1 = root.Symbol,
      splice = arrayProto.splice;

  /* Built-in method references that are verified to be native. */
  var Map = getNative(root, 'Map'),
      nativeCreate = getNative(Object, 'create');

  /** Used to convert symbols to primitives and strings. */
  var symbolProto = Symbol$1 ? Symbol$1.prototype : undefined,
      symbolToString = symbolProto ? symbolProto.toString : undefined;

  /**
   * Creates a hash object.
   *
   * @private
   * @constructor
   * @param {Array} [entries] The key-value pairs to cache.
   */
  function Hash(entries) {
    var index = -1,
        length = entries ? entries.length : 0;

    this.clear();
    while (++index < length) {
      var entry = entries[index];
      this.set(entry[0], entry[1]);
    }
  }

  /**
   * Removes all key-value entries from the hash.
   *
   * @private
   * @name clear
   * @memberOf Hash
   */
  function hashClear() {
    this.__data__ = nativeCreate ? nativeCreate(null) : {};
  }

  /**
   * Removes `key` and its value from the hash.
   *
   * @private
   * @name delete
   * @memberOf Hash
   * @param {Object} hash The hash to modify.
   * @param {string} key The key of the value to remove.
   * @returns {boolean} Returns `true` if the entry was removed, else `false`.
   */
  function hashDelete(key) {
    return this.has(key) && delete this.__data__[key];
  }

  /**
   * Gets the hash value for `key`.
   *
   * @private
   * @name get
   * @memberOf Hash
   * @param {string} key The key of the value to get.
   * @returns {*} Returns the entry value.
   */
  function hashGet(key) {
    var data = this.__data__;
    if (nativeCreate) {
      var result = data[key];
      return result === HASH_UNDEFINED ? undefined : result;
    }
    return hasOwnProperty$1.call(data, key) ? data[key] : undefined;
  }

  /**
   * Checks if a hash value for `key` exists.
   *
   * @private
   * @name has
   * @memberOf Hash
   * @param {string} key The key of the entry to check.
   * @returns {boolean} Returns `true` if an entry for `key` exists, else `false`.
   */
  function hashHas(key) {
    var data = this.__data__;
    return nativeCreate ? data[key] !== undefined : hasOwnProperty$1.call(data, key);
  }

  /**
   * Sets the hash `key` to `value`.
   *
   * @private
   * @name set
   * @memberOf Hash
   * @param {string} key The key of the value to set.
   * @param {*} value The value to set.
   * @returns {Object} Returns the hash instance.
   */
  function hashSet(key, value) {
    var data = this.__data__;
    data[key] = (nativeCreate && value === undefined) ? HASH_UNDEFINED : value;
    return this;
  }

  // Add methods to `Hash`.
  Hash.prototype.clear = hashClear;
  Hash.prototype['delete'] = hashDelete;
  Hash.prototype.get = hashGet;
  Hash.prototype.has = hashHas;
  Hash.prototype.set = hashSet;

  /**
   * Creates an list cache object.
   *
   * @private
   * @constructor
   * @param {Array} [entries] The key-value pairs to cache.
   */
  function ListCache(entries) {
    var index = -1,
        length = entries ? entries.length : 0;

    this.clear();
    while (++index < length) {
      var entry = entries[index];
      this.set(entry[0], entry[1]);
    }
  }

  /**
   * Removes all key-value entries from the list cache.
   *
   * @private
   * @name clear
   * @memberOf ListCache
   */
  function listCacheClear() {
    this.__data__ = [];
  }

  /**
   * Removes `key` and its value from the list cache.
   *
   * @private
   * @name delete
   * @memberOf ListCache
   * @param {string} key The key of the value to remove.
   * @returns {boolean} Returns `true` if the entry was removed, else `false`.
   */
  function listCacheDelete(key) {
    var data = this.__data__,
        index = assocIndexOf(data, key);

    if (index < 0) {
      return false;
    }
    var lastIndex = data.length - 1;
    if (index == lastIndex) {
      data.pop();
    } else {
      splice.call(data, index, 1);
    }
    return true;
  }

  /**
   * Gets the list cache value for `key`.
   *
   * @private
   * @name get
   * @memberOf ListCache
   * @param {string} key The key of the value to get.
   * @returns {*} Returns the entry value.
   */
  function listCacheGet(key) {
    var data = this.__data__,
        index = assocIndexOf(data, key);

    return index < 0 ? undefined : data[index][1];
  }

  /**
   * Checks if a list cache value for `key` exists.
   *
   * @private
   * @name has
   * @memberOf ListCache
   * @param {string} key The key of the entry to check.
   * @returns {boolean} Returns `true` if an entry for `key` exists, else `false`.
   */
  function listCacheHas(key) {
    return assocIndexOf(this.__data__, key) > -1;
  }

  /**
   * Sets the list cache `key` to `value`.
   *
   * @private
   * @name set
   * @memberOf ListCache
   * @param {string} key The key of the value to set.
   * @param {*} value The value to set.
   * @returns {Object} Returns the list cache instance.
   */
  function listCacheSet(key, value) {
    var data = this.__data__,
        index = assocIndexOf(data, key);

    if (index < 0) {
      data.push([key, value]);
    } else {
      data[index][1] = value;
    }
    return this;
  }

  // Add methods to `ListCache`.
  ListCache.prototype.clear = listCacheClear;
  ListCache.prototype['delete'] = listCacheDelete;
  ListCache.prototype.get = listCacheGet;
  ListCache.prototype.has = listCacheHas;
  ListCache.prototype.set = listCacheSet;

  /**
   * Creates a map cache object to store key-value pairs.
   *
   * @private
   * @constructor
   * @param {Array} [entries] The key-value pairs to cache.
   */
  function MapCache(entries) {
    var index = -1,
        length = entries ? entries.length : 0;

    this.clear();
    while (++index < length) {
      var entry = entries[index];
      this.set(entry[0], entry[1]);
    }
  }

  /**
   * Removes all key-value entries from the map.
   *
   * @private
   * @name clear
   * @memberOf MapCache
   */
  function mapCacheClear() {
    this.__data__ = {
      'hash': new Hash,
      'map': new (Map || ListCache),
      'string': new Hash
    };
  }

  /**
   * Removes `key` and its value from the map.
   *
   * @private
   * @name delete
   * @memberOf MapCache
   * @param {string} key The key of the value to remove.
   * @returns {boolean} Returns `true` if the entry was removed, else `false`.
   */
  function mapCacheDelete(key) {
    return getMapData(this, key)['delete'](key);
  }

  /**
   * Gets the map value for `key`.
   *
   * @private
   * @name get
   * @memberOf MapCache
   * @param {string} key The key of the value to get.
   * @returns {*} Returns the entry value.
   */
  function mapCacheGet(key) {
    return getMapData(this, key).get(key);
  }

  /**
   * Checks if a map value for `key` exists.
   *
   * @private
   * @name has
   * @memberOf MapCache
   * @param {string} key The key of the entry to check.
   * @returns {boolean} Returns `true` if an entry for `key` exists, else `false`.
   */
  function mapCacheHas(key) {
    return getMapData(this, key).has(key);
  }

  /**
   * Sets the map `key` to `value`.
   *
   * @private
   * @name set
   * @memberOf MapCache
   * @param {string} key The key of the value to set.
   * @param {*} value The value to set.
   * @returns {Object} Returns the map cache instance.
   */
  function mapCacheSet(key, value) {
    getMapData(this, key).set(key, value);
    return this;
  }

  // Add methods to `MapCache`.
  MapCache.prototype.clear = mapCacheClear;
  MapCache.prototype['delete'] = mapCacheDelete;
  MapCache.prototype.get = mapCacheGet;
  MapCache.prototype.has = mapCacheHas;
  MapCache.prototype.set = mapCacheSet;

  /**
   * Gets the index at which the `key` is found in `array` of key-value pairs.
   *
   * @private
   * @param {Array} array The array to inspect.
   * @param {*} key The key to search for.
   * @returns {number} Returns the index of the matched value, else `-1`.
   */
  function assocIndexOf(array, key) {
    var length = array.length;
    while (length--) {
      if (eq(array[length][0], key)) {
        return length;
      }
    }
    return -1;
  }

  /**
   * The base implementation of `_.get` without support for default values.
   *
   * @private
   * @param {Object} object The object to query.
   * @param {Array|string} path The path of the property to get.
   * @returns {*} Returns the resolved value.
   */
  function baseGet(object, path$$1) {
    path$$1 = isKey(path$$1, object) ? [path$$1] : castPath(path$$1);

    var index = 0,
        length = path$$1.length;

    while (object != null && index < length) {
      object = object[toKey(path$$1[index++])];
    }
    return (index && index == length) ? object : undefined;
  }

  /**
   * The base implementation of `_.isNative` without bad shim checks.
   *
   * @private
   * @param {*} value The value to check.
   * @returns {boolean} Returns `true` if `value` is a native function,
   *  else `false`.
   */
  function baseIsNative(value) {
    if (!isObject(value) || isMasked(value)) {
      return false;
    }
    var pattern = (isFunction(value) || isHostObject(value)) ? reIsNative : reIsHostCtor;
    return pattern.test(toSource(value));
  }

  /**
   * The base implementation of `_.toString` which doesn't convert nullish
   * values to empty strings.
   *
   * @private
   * @param {*} value The value to process.
   * @returns {string} Returns the string.
   */
  function baseToString(value) {
    // Exit early for strings to avoid a performance hit in some environments.
    if (typeof value == 'string') {
      return value;
    }
    if (isSymbol(value)) {
      return symbolToString ? symbolToString.call(value) : '';
    }
    var result = (value + '');
    return (result == '0' && (1 / value) == -INFINITY) ? '-0' : result;
  }

  /**
   * Casts `value` to a path array if it's not one.
   *
   * @private
   * @param {*} value The value to inspect.
   * @returns {Array} Returns the cast property path array.
   */
  function castPath(value) {
    return isArray(value) ? value : stringToPath(value);
  }

  /**
   * Gets the data for `map`.
   *
   * @private
   * @param {Object} map The map to query.
   * @param {string} key The reference key.
   * @returns {*} Returns the map data.
   */
  function getMapData(map, key) {
    var data = map.__data__;
    return isKeyable(key)
      ? data[typeof key == 'string' ? 'string' : 'hash']
      : data.map;
  }

  /**
   * Gets the native function at `key` of `object`.
   *
   * @private
   * @param {Object} object The object to query.
   * @param {string} key The key of the method to get.
   * @returns {*} Returns the function if it's native, else `undefined`.
   */
  function getNative(object, key) {
    var value = getValue(object, key);
    return baseIsNative(value) ? value : undefined;
  }

  /**
   * Checks if `value` is a property name and not a property path.
   *
   * @private
   * @param {*} value The value to check.
   * @param {Object} [object] The object to query keys on.
   * @returns {boolean} Returns `true` if `value` is a property name, else `false`.
   */
  function isKey(value, object) {
    if (isArray(value)) {
      return false;
    }
    var type = typeof value;
    if (type == 'number' || type == 'symbol' || type == 'boolean' ||
        value == null || isSymbol(value)) {
      return true;
    }
    return reIsPlainProp.test(value) || !reIsDeepProp.test(value) ||
      (object != null && value in Object(object));
  }

  /**
   * Checks if `value` is suitable for use as unique object key.
   *
   * @private
   * @param {*} value The value to check.
   * @returns {boolean} Returns `true` if `value` is suitable, else `false`.
   */
  function isKeyable(value) {
    var type = typeof value;
    return (type == 'string' || type == 'number' || type == 'symbol' || type == 'boolean')
      ? (value !== '__proto__')
      : (value === null);
  }

  /**
   * Checks if `func` has its source masked.
   *
   * @private
   * @param {Function} func The function to check.
   * @returns {boolean} Returns `true` if `func` is masked, else `false`.
   */
  function isMasked(func) {
    return !!maskSrcKey && (maskSrcKey in func);
  }

  /**
   * Converts `string` to a property path array.
   *
   * @private
   * @param {string} string The string to convert.
   * @returns {Array} Returns the property path array.
   */
  var stringToPath = memoize(function(string) {
    string = toString(string);

    var result = [];
    if (reLeadingDot.test(string)) {
      result.push('');
    }
    string.replace(rePropName, function(match, number, quote, string) {
      result.push(quote ? string.replace(reEscapeChar, '$1') : (number || match));
    });
    return result;
  });

  /**
   * Converts `value` to a string key if it's not a string or symbol.
   *
   * @private
   * @param {*} value The value to inspect.
   * @returns {string|symbol} Returns the key.
   */
  function toKey(value) {
    if (typeof value == 'string' || isSymbol(value)) {
      return value;
    }
    var result = (value + '');
    return (result == '0' && (1 / value) == -INFINITY) ? '-0' : result;
  }

  /**
   * Converts `func` to its source code.
   *
   * @private
   * @param {Function} func The function to process.
   * @returns {string} Returns the source code.
   */
  function toSource(func) {
    if (func != null) {
      try {
        return funcToString.call(func);
      } catch (e) {}
      try {
        return (func + '');
      } catch (e) {}
    }
    return '';
  }

  /**
   * Creates a function that memoizes the result of `func`. If `resolver` is
   * provided, it determines the cache key for storing the result based on the
   * arguments provided to the memoized function. By default, the first argument
   * provided to the memoized function is used as the map cache key. The `func`
   * is invoked with the `this` binding of the memoized function.
   *
   * **Note:** The cache is exposed as the `cache` property on the memoized
   * function. Its creation may be customized by replacing the `_.memoize.Cache`
   * constructor with one whose instances implement the
   * [`Map`](http://ecma-international.org/ecma-262/7.0/#sec-properties-of-the-map-prototype-object)
   * method interface of `delete`, `get`, `has`, and `set`.
   *
   * @static
   * @memberOf _
   * @since 0.1.0
   * @category Function
   * @param {Function} func The function to have its output memoized.
   * @param {Function} [resolver] The function to resolve the cache key.
   * @returns {Function} Returns the new memoized function.
   * @example
   *
   * var object = { 'a': 1, 'b': 2 };
   * var other = { 'c': 3, 'd': 4 };
   *
   * var values = _.memoize(_.values);
   * values(object);
   * // => [1, 2]
   *
   * values(other);
   * // => [3, 4]
   *
   * object.a = 2;
   * values(object);
   * // => [1, 2]
   *
   * // Modify the result cache.
   * values.cache.set(object, ['a', 'b']);
   * values(object);
   * // => ['a', 'b']
   *
   * // Replace `_.memoize.Cache`.
   * _.memoize.Cache = WeakMap;
   */
  function memoize(func, resolver) {
    if (typeof func != 'function' || (resolver && typeof resolver != 'function')) {
      throw new TypeError(FUNC_ERROR_TEXT);
    }
    var memoized = function() {
      var args = arguments,
          key = resolver ? resolver.apply(this, args) : args[0],
          cache = memoized.cache;

      if (cache.has(key)) {
        return cache.get(key);
      }
      var result = func.apply(this, args);
      memoized.cache = cache.set(key, result);
      return result;
    };
    memoized.cache = new (memoize.Cache || MapCache);
    return memoized;
  }

  // Assign cache to `_.memoize`.
  memoize.Cache = MapCache;

  /**
   * Performs a
   * [`SameValueZero`](http://ecma-international.org/ecma-262/7.0/#sec-samevaluezero)
   * comparison between two values to determine if they are equivalent.
   *
   * @static
   * @memberOf _
   * @since 4.0.0
   * @category Lang
   * @param {*} value The value to compare.
   * @param {*} other The other value to compare.
   * @returns {boolean} Returns `true` if the values are equivalent, else `false`.
   * @example
   *
   * var object = { 'a': 1 };
   * var other = { 'a': 1 };
   *
   * _.eq(object, object);
   * // => true
   *
   * _.eq(object, other);
   * // => false
   *
   * _.eq('a', 'a');
   * // => true
   *
   * _.eq('a', Object('a'));
   * // => false
   *
   * _.eq(NaN, NaN);
   * // => true
   */
  function eq(value, other) {
    return value === other || (value !== value && other !== other);
  }

  /**
   * Checks if `value` is classified as an `Array` object.
   *
   * @static
   * @memberOf _
   * @since 0.1.0
   * @category Lang
   * @param {*} value The value to check.
   * @returns {boolean} Returns `true` if `value` is an array, else `false`.
   * @example
   *
   * _.isArray([1, 2, 3]);
   * // => true
   *
   * _.isArray(document.body.children);
   * // => false
   *
   * _.isArray('abc');
   * // => false
   *
   * _.isArray(_.noop);
   * // => false
   */
  var isArray = Array.isArray;

  /**
   * Checks if `value` is classified as a `Function` object.
   *
   * @static
   * @memberOf _
   * @since 0.1.0
   * @category Lang
   * @param {*} value The value to check.
   * @returns {boolean} Returns `true` if `value` is a function, else `false`.
   * @example
   *
   * _.isFunction(_);
   * // => true
   *
   * _.isFunction(/abc/);
   * // => false
   */
  function isFunction(value) {
    // The use of `Object#toString` avoids issues with the `typeof` operator
    // in Safari 8-9 which returns 'object' for typed array and other constructors.
    var tag = isObject(value) ? objectToString.call(value) : '';
    return tag == funcTag || tag == genTag;
  }

  /**
   * Checks if `value` is the
   * [language type](http://www.ecma-international.org/ecma-262/7.0/#sec-ecmascript-language-types)
   * of `Object`. (e.g. arrays, functions, objects, regexes, `new Number(0)`, and `new String('')`)
   *
   * @static
   * @memberOf _
   * @since 0.1.0
   * @category Lang
   * @param {*} value The value to check.
   * @returns {boolean} Returns `true` if `value` is an object, else `false`.
   * @example
   *
   * _.isObject({});
   * // => true
   *
   * _.isObject([1, 2, 3]);
   * // => true
   *
   * _.isObject(_.noop);
   * // => true
   *
   * _.isObject(null);
   * // => false
   */
  function isObject(value) {
    var type = typeof value;
    return !!value && (type == 'object' || type == 'function');
  }

  /**
   * Checks if `value` is object-like. A value is object-like if it's not `null`
   * and has a `typeof` result of "object".
   *
   * @static
   * @memberOf _
   * @since 4.0.0
   * @category Lang
   * @param {*} value The value to check.
   * @returns {boolean} Returns `true` if `value` is object-like, else `false`.
   * @example
   *
   * _.isObjectLike({});
   * // => true
   *
   * _.isObjectLike([1, 2, 3]);
   * // => true
   *
   * _.isObjectLike(_.noop);
   * // => false
   *
   * _.isObjectLike(null);
   * // => false
   */
  function isObjectLike(value) {
    return !!value && typeof value == 'object';
  }

  /**
   * Checks if `value` is classified as a `Symbol` primitive or object.
   *
   * @static
   * @memberOf _
   * @since 4.0.0
   * @category Lang
   * @param {*} value The value to check.
   * @returns {boolean} Returns `true` if `value` is a symbol, else `false`.
   * @example
   *
   * _.isSymbol(Symbol.iterator);
   * // => true
   *
   * _.isSymbol('abc');
   * // => false
   */
  function isSymbol(value) {
    return typeof value == 'symbol' ||
      (isObjectLike(value) && objectToString.call(value) == symbolTag);
  }

  /**
   * Converts `value` to a string. An empty string is returned for `null`
   * and `undefined` values. The sign of `-0` is preserved.
   *
   * @static
   * @memberOf _
   * @since 4.0.0
   * @category Lang
   * @param {*} value The value to process.
   * @returns {string} Returns the string.
   * @example
   *
   * _.toString(null);
   * // => ''
   *
   * _.toString(-0);
   * // => '-0'
   *
   * _.toString([1, 2, 3]);
   * // => '1,2,3'
   */
  function toString(value) {
    return value == null ? '' : baseToString(value);
  }

  /**
   * Gets the value at `path` of `object`. If the resolved value is
   * `undefined`, the `defaultValue` is returned in its place.
   *
   * @static
   * @memberOf _
   * @since 3.7.0
   * @category Object
   * @param {Object} object The object to query.
   * @param {Array|string} path The path of the property to get.
   * @param {*} [defaultValue] The value returned for `undefined` resolved values.
   * @returns {*} Returns the resolved value.
   * @example
   *
   * var object = { 'a': [{ 'b': { 'c': 3 } }] };
   *
   * _.get(object, 'a[0].b.c');
   * // => 3
   *
   * _.get(object, ['a', '0', 'b', 'c']);
   * // => 3
   *
   * _.get(object, 'a.b.c', 'default');
   * // => 'default'
   */
  function get(object, path$$1, defaultValue) {
    var result = object == null ? undefined : baseGet(object, path$$1);
    return result === undefined ? defaultValue : result;
  }

  var lodash_get = get;

  /**
   * lodash (Custom Build) <https://lodash.com/>
   * Build: `lodash modularize exports="npm" -o ./`
   * Copyright jQuery Foundation and other contributors <https://jquery.org/>
   * Released under MIT license <https://lodash.com/license>
   * Based on Underscore.js 1.8.3 <http://underscorejs.org/LICENSE>
   * Copyright Jeremy Ashkenas, DocumentCloud and Investigative Reporters & Editors
   */

  /** Used as references for various `Number` constants. */
  var INFINITY$1 = 1 / 0;

  /** `Object#toString` result references. */
  var symbolTag$1 = '[object Symbol]';

  /** Used to match leading and trailing whitespace. */
  var reTrimStart = /^\s+/;

  /** Used to compose unicode character classes. */
  var rsAstralRange = '\\ud800-\\udfff',
      rsComboMarksRange = '\\u0300-\\u036f\\ufe20-\\ufe23',
      rsComboSymbolsRange = '\\u20d0-\\u20f0',
      rsVarRange = '\\ufe0e\\ufe0f';

  /** Used to compose unicode capture groups. */
  var rsAstral = '[' + rsAstralRange + ']',
      rsCombo = '[' + rsComboMarksRange + rsComboSymbolsRange + ']',
      rsFitz = '\\ud83c[\\udffb-\\udfff]',
      rsModifier = '(?:' + rsCombo + '|' + rsFitz + ')',
      rsNonAstral = '[^' + rsAstralRange + ']',
      rsRegional = '(?:\\ud83c[\\udde6-\\uddff]){2}',
      rsSurrPair = '[\\ud800-\\udbff][\\udc00-\\udfff]',
      rsZWJ = '\\u200d';

  /** Used to compose unicode regexes. */
  var reOptMod = rsModifier + '?',
      rsOptVar = '[' + rsVarRange + ']?',
      rsOptJoin = '(?:' + rsZWJ + '(?:' + [rsNonAstral, rsRegional, rsSurrPair].join('|') + ')' + rsOptVar + reOptMod + ')*',
      rsSeq = rsOptVar + reOptMod + rsOptJoin,
      rsSymbol = '(?:' + [rsNonAstral + rsCombo + '?', rsCombo, rsRegional, rsSurrPair, rsAstral].join('|') + ')';

  /** Used to match [string symbols](https://mathiasbynens.be/notes/javascript-unicode). */
  var reUnicode = RegExp(rsFitz + '(?=' + rsFitz + ')|' + rsSymbol + rsSeq, 'g');

  /** Used to detect strings with [zero-width joiners or code points from the astral planes](http://eev.ee/blog/2015/09/12/dark-corners-of-unicode/). */
  var reHasUnicode = RegExp('[' + rsZWJ + rsAstralRange  + rsComboMarksRange + rsComboSymbolsRange + rsVarRange + ']');

  /** Detect free variable `global` from Node.js. */
  var freeGlobal$1 = typeof commonjsGlobal == 'object' && commonjsGlobal && commonjsGlobal.Object === Object && commonjsGlobal;

  /** Detect free variable `self`. */
  var freeSelf$1 = typeof self == 'object' && self && self.Object === Object && self;

  /** Used as a reference to the global object. */
  var root$1 = freeGlobal$1 || freeSelf$1 || Function('return this')();

  /**
   * Converts an ASCII `string` to an array.
   *
   * @private
   * @param {string} string The string to convert.
   * @returns {Array} Returns the converted array.
   */
  function asciiToArray(string) {
    return string.split('');
  }

  /**
   * The base implementation of `_.findIndex` and `_.findLastIndex` without
   * support for iteratee shorthands.
   *
   * @private
   * @param {Array} array The array to inspect.
   * @param {Function} predicate The function invoked per iteration.
   * @param {number} fromIndex The index to search from.
   * @param {boolean} [fromRight] Specify iterating from right to left.
   * @returns {number} Returns the index of the matched value, else `-1`.
   */
  function baseFindIndex(array, predicate, fromIndex, fromRight) {
    var length = array.length,
        index = fromIndex + (fromRight ? 1 : -1);

    while ((fromRight ? index-- : ++index < length)) {
      if (predicate(array[index], index, array)) {
        return index;
      }
    }
    return -1;
  }

  /**
   * The base implementation of `_.indexOf` without `fromIndex` bounds checks.
   *
   * @private
   * @param {Array} array The array to inspect.
   * @param {*} value The value to search for.
   * @param {number} fromIndex The index to search from.
   * @returns {number} Returns the index of the matched value, else `-1`.
   */
  function baseIndexOf(array, value, fromIndex) {
    if (value !== value) {
      return baseFindIndex(array, baseIsNaN, fromIndex);
    }
    var index = fromIndex - 1,
        length = array.length;

    while (++index < length) {
      if (array[index] === value) {
        return index;
      }
    }
    return -1;
  }

  /**
   * The base implementation of `_.isNaN` without support for number objects.
   *
   * @private
   * @param {*} value The value to check.
   * @returns {boolean} Returns `true` if `value` is `NaN`, else `false`.
   */
  function baseIsNaN(value) {
    return value !== value;
  }

  /**
   * Used by `_.trim` and `_.trimStart` to get the index of the first string symbol
   * that is not found in the character symbols.
   *
   * @private
   * @param {Array} strSymbols The string symbols to inspect.
   * @param {Array} chrSymbols The character symbols to find.
   * @returns {number} Returns the index of the first unmatched string symbol.
   */
  function charsStartIndex(strSymbols, chrSymbols) {
    var index = -1,
        length = strSymbols.length;

    while (++index < length && baseIndexOf(chrSymbols, strSymbols[index], 0) > -1) {}
    return index;
  }

  /**
   * Checks if `string` contains Unicode symbols.
   *
   * @private
   * @param {string} string The string to inspect.
   * @returns {boolean} Returns `true` if a symbol is found, else `false`.
   */
  function hasUnicode(string) {
    return reHasUnicode.test(string);
  }

  /**
   * Converts `string` to an array.
   *
   * @private
   * @param {string} string The string to convert.
   * @returns {Array} Returns the converted array.
   */
  function stringToArray(string) {
    return hasUnicode(string)
      ? unicodeToArray(string)
      : asciiToArray(string);
  }

  /**
   * Converts a Unicode `string` to an array.
   *
   * @private
   * @param {string} string The string to convert.
   * @returns {Array} Returns the converted array.
   */
  function unicodeToArray(string) {
    return string.match(reUnicode) || [];
  }

  /** Used for built-in method references. */
  var objectProto$1 = Object.prototype;

  /**
   * Used to resolve the
   * [`toStringTag`](http://ecma-international.org/ecma-262/7.0/#sec-object.prototype.tostring)
   * of values.
   */
  var objectToString$1 = objectProto$1.toString;

  /** Built-in value references. */
  var Symbol$2 = root$1.Symbol;

  /** Used to convert symbols to primitives and strings. */
  var symbolProto$1 = Symbol$2 ? Symbol$2.prototype : undefined,
      symbolToString$1 = symbolProto$1 ? symbolProto$1.toString : undefined;

  /**
   * The base implementation of `_.slice` without an iteratee call guard.
   *
   * @private
   * @param {Array} array The array to slice.
   * @param {number} [start=0] The start position.
   * @param {number} [end=array.length] The end position.
   * @returns {Array} Returns the slice of `array`.
   */
  function baseSlice(array, start, end) {
    var index = -1,
        length = array.length;

    if (start < 0) {
      start = -start > length ? 0 : (length + start);
    }
    end = end > length ? length : end;
    if (end < 0) {
      end += length;
    }
    length = start > end ? 0 : ((end - start) >>> 0);
    start >>>= 0;

    var result = Array(length);
    while (++index < length) {
      result[index] = array[index + start];
    }
    return result;
  }

  /**
   * The base implementation of `_.toString` which doesn't convert nullish
   * values to empty strings.
   *
   * @private
   * @param {*} value The value to process.
   * @returns {string} Returns the string.
   */
  function baseToString$1(value) {
    // Exit early for strings to avoid a performance hit in some environments.
    if (typeof value == 'string') {
      return value;
    }
    if (isSymbol$1(value)) {
      return symbolToString$1 ? symbolToString$1.call(value) : '';
    }
    var result = (value + '');
    return (result == '0' && (1 / value) == -INFINITY$1) ? '-0' : result;
  }

  /**
   * Casts `array` to a slice if it's needed.
   *
   * @private
   * @param {Array} array The array to inspect.
   * @param {number} start The start position.
   * @param {number} [end=array.length] The end position.
   * @returns {Array} Returns the cast slice.
   */
  function castSlice(array, start, end) {
    var length = array.length;
    end = end === undefined ? length : end;
    return (!start && end >= length) ? array : baseSlice(array, start, end);
  }

  /**
   * Checks if `value` is object-like. A value is object-like if it's not `null`
   * and has a `typeof` result of "object".
   *
   * @static
   * @memberOf _
   * @since 4.0.0
   * @category Lang
   * @param {*} value The value to check.
   * @returns {boolean} Returns `true` if `value` is object-like, else `false`.
   * @example
   *
   * _.isObjectLike({});
   * // => true
   *
   * _.isObjectLike([1, 2, 3]);
   * // => true
   *
   * _.isObjectLike(_.noop);
   * // => false
   *
   * _.isObjectLike(null);
   * // => false
   */
  function isObjectLike$1(value) {
    return !!value && typeof value == 'object';
  }

  /**
   * Checks if `value` is classified as a `Symbol` primitive or object.
   *
   * @static
   * @memberOf _
   * @since 4.0.0
   * @category Lang
   * @param {*} value The value to check.
   * @returns {boolean} Returns `true` if `value` is a symbol, else `false`.
   * @example
   *
   * _.isSymbol(Symbol.iterator);
   * // => true
   *
   * _.isSymbol('abc');
   * // => false
   */
  function isSymbol$1(value) {
    return typeof value == 'symbol' ||
      (isObjectLike$1(value) && objectToString$1.call(value) == symbolTag$1);
  }

  /**
   * Converts `value` to a string. An empty string is returned for `null`
   * and `undefined` values. The sign of `-0` is preserved.
   *
   * @static
   * @memberOf _
   * @since 4.0.0
   * @category Lang
   * @param {*} value The value to process.
   * @returns {string} Returns the string.
   * @example
   *
   * _.toString(null);
   * // => ''
   *
   * _.toString(-0);
   * // => '-0'
   *
   * _.toString([1, 2, 3]);
   * // => '1,2,3'
   */
  function toString$1(value) {
    return value == null ? '' : baseToString$1(value);
  }

  /**
   * Removes leading whitespace or specified characters from `string`.
   *
   * @static
   * @memberOf _
   * @since 4.0.0
   * @category String
   * @param {string} [string=''] The string to trim.
   * @param {string} [chars=whitespace] The characters to trim.
   * @param- {Object} [guard] Enables use as an iteratee for methods like `_.map`.
   * @returns {string} Returns the trimmed string.
   * @example
   *
   * _.trimStart('  abc  ');
   * // => 'abc  '
   *
   * _.trimStart('-_-abc-_-', '_-');
   * // => 'abc-_-'
   */
  function trimStart(string, chars, guard) {
    string = toString$1(string);
    if (string && (guard || chars === undefined)) {
      return string.replace(reTrimStart, '');
    }
    if (!string || !(chars = baseToString$1(chars))) {
      return string;
    }
    var strSymbols = stringToArray(string),
        start = charsStartIndex(strSymbols, stringToArray(chars));

    return castSlice(strSymbols, start).join('');
  }

  var lodash_trimstart = trimStart;

  /**
   * lodash (Custom Build) <https://lodash.com/>
   * Build: `lodash modularize exports="npm" -o ./`
   * Copyright jQuery Foundation and other contributors <https://jquery.org/>
   * Released under MIT license <https://lodash.com/license>
   * Based on Underscore.js 1.8.3 <http://underscorejs.org/LICENSE>
   * Copyright Jeremy Ashkenas, DocumentCloud and Investigative Reporters & Editors
   */

  /** Used as the `TypeError` message for "Functions" methods. */
  var FUNC_ERROR_TEXT$1 = 'Expected a function';

  /** Used to stand-in for `undefined` hash values. */
  var HASH_UNDEFINED$1 = '__lodash_hash_undefined__';

  /** Used as references for various `Number` constants. */
  var INFINITY$2 = 1 / 0,
      MAX_SAFE_INTEGER = 9007199254740991;

  /** `Object#toString` result references. */
  var funcTag$1 = '[object Function]',
      genTag$1 = '[object GeneratorFunction]',
      symbolTag$2 = '[object Symbol]';

  /** Used to match property names within property paths. */
  var reIsDeepProp$1 = /\.|\[(?:[^[\]]*|(["'])(?:(?!\1)[^\\]|\\.)*?\1)\]/,
      reIsPlainProp$1 = /^\w*$/,
      reLeadingDot$1 = /^\./,
      rePropName$1 = /[^.[\]]+|\[(?:(-?\d+(?:\.\d+)?)|(["'])((?:(?!\2)[^\\]|\\.)*?)\2)\]|(?=(?:\.|\[\])(?:\.|\[\]|$))/g;

  /**
   * Used to match `RegExp`
   * [syntax characters](http://ecma-international.org/ecma-262/7.0/#sec-patterns).
   */
  var reRegExpChar$1 = /[\\^$.*+?()[\]{}|]/g;

  /** Used to match backslashes in property paths. */
  var reEscapeChar$1 = /\\(\\)?/g;

  /** Used to detect host constructors (Safari). */
  var reIsHostCtor$1 = /^\[object .+?Constructor\]$/;

  /** Used to detect unsigned integer values. */
  var reIsUint = /^(?:0|[1-9]\d*)$/;

  /** Detect free variable `global` from Node.js. */
  var freeGlobal$2 = typeof commonjsGlobal == 'object' && commonjsGlobal && commonjsGlobal.Object === Object && commonjsGlobal;

  /** Detect free variable `self`. */
  var freeSelf$2 = typeof self == 'object' && self && self.Object === Object && self;

  /** Used as a reference to the global object. */
  var root$2 = freeGlobal$2 || freeSelf$2 || Function('return this')();

  /**
   * Gets the value at `key` of `object`.
   *
   * @private
   * @param {Object} [object] The object to query.
   * @param {string} key The key of the property to get.
   * @returns {*} Returns the property value.
   */
  function getValue$1(object, key) {
    return object == null ? undefined : object[key];
  }

  /**
   * Checks if `value` is a host object in IE < 9.
   *
   * @private
   * @param {*} value The value to check.
   * @returns {boolean} Returns `true` if `value` is a host object, else `false`.
   */
  function isHostObject$1(value) {
    // Many host objects are `Object` objects that can coerce to strings
    // despite having improperly defined `toString` methods.
    var result = false;
    if (value != null && typeof value.toString != 'function') {
      try {
        result = !!(value + '');
      } catch (e) {}
    }
    return result;
  }

  /** Used for built-in method references. */
  var arrayProto$1 = Array.prototype,
      funcProto$1 = Function.prototype,
      objectProto$2 = Object.prototype;

  /** Used to detect overreaching core-js shims. */
  var coreJsData$1 = root$2['__core-js_shared__'];

  /** Used to detect methods masquerading as native. */
  var maskSrcKey$1 = (function() {
    var uid = /[^.]+$/.exec(coreJsData$1 && coreJsData$1.keys && coreJsData$1.keys.IE_PROTO || '');
    return uid ? ('Symbol(src)_1.' + uid) : '';
  }());

  /** Used to resolve the decompiled source of functions. */
  var funcToString$1 = funcProto$1.toString;

  /** Used to check objects for own properties. */
  var hasOwnProperty$2 = objectProto$2.hasOwnProperty;

  /**
   * Used to resolve the
   * [`toStringTag`](http://ecma-international.org/ecma-262/7.0/#sec-object.prototype.tostring)
   * of values.
   */
  var objectToString$2 = objectProto$2.toString;

  /** Used to detect if a method is native. */
  var reIsNative$1 = RegExp('^' +
    funcToString$1.call(hasOwnProperty$2).replace(reRegExpChar$1, '\\$&')
    .replace(/hasOwnProperty|(function).*?(?=\\\()| for .+?(?=\\\])/g, '$1.*?') + '$'
  );

  /** Built-in value references. */
  var Symbol$3 = root$2.Symbol,
      splice$1 = arrayProto$1.splice;

  /* Built-in method references that are verified to be native. */
  var Map$1 = getNative$1(root$2, 'Map'),
      nativeCreate$1 = getNative$1(Object, 'create');

  /** Used to convert symbols to primitives and strings. */
  var symbolProto$2 = Symbol$3 ? Symbol$3.prototype : undefined,
      symbolToString$2 = symbolProto$2 ? symbolProto$2.toString : undefined;

  /**
   * Creates a hash object.
   *
   * @private
   * @constructor
   * @param {Array} [entries] The key-value pairs to cache.
   */
  function Hash$1(entries) {
    var index = -1,
        length = entries ? entries.length : 0;

    this.clear();
    while (++index < length) {
      var entry = entries[index];
      this.set(entry[0], entry[1]);
    }
  }

  /**
   * Removes all key-value entries from the hash.
   *
   * @private
   * @name clear
   * @memberOf Hash
   */
  function hashClear$1() {
    this.__data__ = nativeCreate$1 ? nativeCreate$1(null) : {};
  }

  /**
   * Removes `key` and its value from the hash.
   *
   * @private
   * @name delete
   * @memberOf Hash
   * @param {Object} hash The hash to modify.
   * @param {string} key The key of the value to remove.
   * @returns {boolean} Returns `true` if the entry was removed, else `false`.
   */
  function hashDelete$1(key) {
    return this.has(key) && delete this.__data__[key];
  }

  /**
   * Gets the hash value for `key`.
   *
   * @private
   * @name get
   * @memberOf Hash
   * @param {string} key The key of the value to get.
   * @returns {*} Returns the entry value.
   */
  function hashGet$1(key) {
    var data = this.__data__;
    if (nativeCreate$1) {
      var result = data[key];
      return result === HASH_UNDEFINED$1 ? undefined : result;
    }
    return hasOwnProperty$2.call(data, key) ? data[key] : undefined;
  }

  /**
   * Checks if a hash value for `key` exists.
   *
   * @private
   * @name has
   * @memberOf Hash
   * @param {string} key The key of the entry to check.
   * @returns {boolean} Returns `true` if an entry for `key` exists, else `false`.
   */
  function hashHas$1(key) {
    var data = this.__data__;
    return nativeCreate$1 ? data[key] !== undefined : hasOwnProperty$2.call(data, key);
  }

  /**
   * Sets the hash `key` to `value`.
   *
   * @private
   * @name set
   * @memberOf Hash
   * @param {string} key The key of the value to set.
   * @param {*} value The value to set.
   * @returns {Object} Returns the hash instance.
   */
  function hashSet$1(key, value) {
    var data = this.__data__;
    data[key] = (nativeCreate$1 && value === undefined) ? HASH_UNDEFINED$1 : value;
    return this;
  }

  // Add methods to `Hash`.
  Hash$1.prototype.clear = hashClear$1;
  Hash$1.prototype['delete'] = hashDelete$1;
  Hash$1.prototype.get = hashGet$1;
  Hash$1.prototype.has = hashHas$1;
  Hash$1.prototype.set = hashSet$1;

  /**
   * Creates an list cache object.
   *
   * @private
   * @constructor
   * @param {Array} [entries] The key-value pairs to cache.
   */
  function ListCache$1(entries) {
    var index = -1,
        length = entries ? entries.length : 0;

    this.clear();
    while (++index < length) {
      var entry = entries[index];
      this.set(entry[0], entry[1]);
    }
  }

  /**
   * Removes all key-value entries from the list cache.
   *
   * @private
   * @name clear
   * @memberOf ListCache
   */
  function listCacheClear$1() {
    this.__data__ = [];
  }

  /**
   * Removes `key` and its value from the list cache.
   *
   * @private
   * @name delete
   * @memberOf ListCache
   * @param {string} key The key of the value to remove.
   * @returns {boolean} Returns `true` if the entry was removed, else `false`.
   */
  function listCacheDelete$1(key) {
    var data = this.__data__,
        index = assocIndexOf$1(data, key);

    if (index < 0) {
      return false;
    }
    var lastIndex = data.length - 1;
    if (index == lastIndex) {
      data.pop();
    } else {
      splice$1.call(data, index, 1);
    }
    return true;
  }

  /**
   * Gets the list cache value for `key`.
   *
   * @private
   * @name get
   * @memberOf ListCache
   * @param {string} key The key of the value to get.
   * @returns {*} Returns the entry value.
   */
  function listCacheGet$1(key) {
    var data = this.__data__,
        index = assocIndexOf$1(data, key);

    return index < 0 ? undefined : data[index][1];
  }

  /**
   * Checks if a list cache value for `key` exists.
   *
   * @private
   * @name has
   * @memberOf ListCache
   * @param {string} key The key of the entry to check.
   * @returns {boolean} Returns `true` if an entry for `key` exists, else `false`.
   */
  function listCacheHas$1(key) {
    return assocIndexOf$1(this.__data__, key) > -1;
  }

  /**
   * Sets the list cache `key` to `value`.
   *
   * @private
   * @name set
   * @memberOf ListCache
   * @param {string} key The key of the value to set.
   * @param {*} value The value to set.
   * @returns {Object} Returns the list cache instance.
   */
  function listCacheSet$1(key, value) {
    var data = this.__data__,
        index = assocIndexOf$1(data, key);

    if (index < 0) {
      data.push([key, value]);
    } else {
      data[index][1] = value;
    }
    return this;
  }

  // Add methods to `ListCache`.
  ListCache$1.prototype.clear = listCacheClear$1;
  ListCache$1.prototype['delete'] = listCacheDelete$1;
  ListCache$1.prototype.get = listCacheGet$1;
  ListCache$1.prototype.has = listCacheHas$1;
  ListCache$1.prototype.set = listCacheSet$1;

  /**
   * Creates a map cache object to store key-value pairs.
   *
   * @private
   * @constructor
   * @param {Array} [entries] The key-value pairs to cache.
   */
  function MapCache$1(entries) {
    var index = -1,
        length = entries ? entries.length : 0;

    this.clear();
    while (++index < length) {
      var entry = entries[index];
      this.set(entry[0], entry[1]);
    }
  }

  /**
   * Removes all key-value entries from the map.
   *
   * @private
   * @name clear
   * @memberOf MapCache
   */
  function mapCacheClear$1() {
    this.__data__ = {
      'hash': new Hash$1,
      'map': new (Map$1 || ListCache$1),
      'string': new Hash$1
    };
  }

  /**
   * Removes `key` and its value from the map.
   *
   * @private
   * @name delete
   * @memberOf MapCache
   * @param {string} key The key of the value to remove.
   * @returns {boolean} Returns `true` if the entry was removed, else `false`.
   */
  function mapCacheDelete$1(key) {
    return getMapData$1(this, key)['delete'](key);
  }

  /**
   * Gets the map value for `key`.
   *
   * @private
   * @name get
   * @memberOf MapCache
   * @param {string} key The key of the value to get.
   * @returns {*} Returns the entry value.
   */
  function mapCacheGet$1(key) {
    return getMapData$1(this, key).get(key);
  }

  /**
   * Checks if a map value for `key` exists.
   *
   * @private
   * @name has
   * @memberOf MapCache
   * @param {string} key The key of the entry to check.
   * @returns {boolean} Returns `true` if an entry for `key` exists, else `false`.
   */
  function mapCacheHas$1(key) {
    return getMapData$1(this, key).has(key);
  }

  /**
   * Sets the map `key` to `value`.
   *
   * @private
   * @name set
   * @memberOf MapCache
   * @param {string} key The key of the value to set.
   * @param {*} value The value to set.
   * @returns {Object} Returns the map cache instance.
   */
  function mapCacheSet$1(key, value) {
    getMapData$1(this, key).set(key, value);
    return this;
  }

  // Add methods to `MapCache`.
  MapCache$1.prototype.clear = mapCacheClear$1;
  MapCache$1.prototype['delete'] = mapCacheDelete$1;
  MapCache$1.prototype.get = mapCacheGet$1;
  MapCache$1.prototype.has = mapCacheHas$1;
  MapCache$1.prototype.set = mapCacheSet$1;

  /**
   * Assigns `value` to `key` of `object` if the existing value is not equivalent
   * using [`SameValueZero`](http://ecma-international.org/ecma-262/7.0/#sec-samevaluezero)
   * for equality comparisons.
   *
   * @private
   * @param {Object} object The object to modify.
   * @param {string} key The key of the property to assign.
   * @param {*} value The value to assign.
   */
  function assignValue(object, key, value) {
    var objValue = object[key];
    if (!(hasOwnProperty$2.call(object, key) && eq$1(objValue, value)) ||
        (value === undefined && !(key in object))) {
      object[key] = value;
    }
  }

  /**
   * Gets the index at which the `key` is found in `array` of key-value pairs.
   *
   * @private
   * @param {Array} array The array to inspect.
   * @param {*} key The key to search for.
   * @returns {number} Returns the index of the matched value, else `-1`.
   */
  function assocIndexOf$1(array, key) {
    var length = array.length;
    while (length--) {
      if (eq$1(array[length][0], key)) {
        return length;
      }
    }
    return -1;
  }

  /**
   * The base implementation of `_.isNative` without bad shim checks.
   *
   * @private
   * @param {*} value The value to check.
   * @returns {boolean} Returns `true` if `value` is a native function,
   *  else `false`.
   */
  function baseIsNative$1(value) {
    if (!isObject$1(value) || isMasked$1(value)) {
      return false;
    }
    var pattern = (isFunction$1(value) || isHostObject$1(value)) ? reIsNative$1 : reIsHostCtor$1;
    return pattern.test(toSource$1(value));
  }

  /**
   * The base implementation of `_.set`.
   *
   * @private
   * @param {Object} object The object to modify.
   * @param {Array|string} path The path of the property to set.
   * @param {*} value The value to set.
   * @param {Function} [customizer] The function to customize path creation.
   * @returns {Object} Returns `object`.
   */
  function baseSet(object, path$$1, value, customizer) {
    if (!isObject$1(object)) {
      return object;
    }
    path$$1 = isKey$1(path$$1, object) ? [path$$1] : castPath$1(path$$1);

    var index = -1,
        length = path$$1.length,
        lastIndex = length - 1,
        nested = object;

    while (nested != null && ++index < length) {
      var key = toKey$1(path$$1[index]),
          newValue = value;

      if (index != lastIndex) {
        var objValue = nested[key];
        newValue = customizer ? customizer(objValue, key, nested) : undefined;
        if (newValue === undefined) {
          newValue = isObject$1(objValue)
            ? objValue
            : (isIndex(path$$1[index + 1]) ? [] : {});
        }
      }
      assignValue(nested, key, newValue);
      nested = nested[key];
    }
    return object;
  }

  /**
   * The base implementation of `_.toString` which doesn't convert nullish
   * values to empty strings.
   *
   * @private
   * @param {*} value The value to process.
   * @returns {string} Returns the string.
   */
  function baseToString$2(value) {
    // Exit early for strings to avoid a performance hit in some environments.
    if (typeof value == 'string') {
      return value;
    }
    if (isSymbol$2(value)) {
      return symbolToString$2 ? symbolToString$2.call(value) : '';
    }
    var result = (value + '');
    return (result == '0' && (1 / value) == -INFINITY$2) ? '-0' : result;
  }

  /**
   * Casts `value` to a path array if it's not one.
   *
   * @private
   * @param {*} value The value to inspect.
   * @returns {Array} Returns the cast property path array.
   */
  function castPath$1(value) {
    return isArray$1(value) ? value : stringToPath$1(value);
  }

  /**
   * Gets the data for `map`.
   *
   * @private
   * @param {Object} map The map to query.
   * @param {string} key The reference key.
   * @returns {*} Returns the map data.
   */
  function getMapData$1(map, key) {
    var data = map.__data__;
    return isKeyable$1(key)
      ? data[typeof key == 'string' ? 'string' : 'hash']
      : data.map;
  }

  /**
   * Gets the native function at `key` of `object`.
   *
   * @private
   * @param {Object} object The object to query.
   * @param {string} key The key of the method to get.
   * @returns {*} Returns the function if it's native, else `undefined`.
   */
  function getNative$1(object, key) {
    var value = getValue$1(object, key);
    return baseIsNative$1(value) ? value : undefined;
  }

  /**
   * Checks if `value` is a valid array-like index.
   *
   * @private
   * @param {*} value The value to check.
   * @param {number} [length=MAX_SAFE_INTEGER] The upper bounds of a valid index.
   * @returns {boolean} Returns `true` if `value` is a valid index, else `false`.
   */
  function isIndex(value, length) {
    length = length == null ? MAX_SAFE_INTEGER : length;
    return !!length &&
      (typeof value == 'number' || reIsUint.test(value)) &&
      (value > -1 && value % 1 == 0 && value < length);
  }

  /**
   * Checks if `value` is a property name and not a property path.
   *
   * @private
   * @param {*} value The value to check.
   * @param {Object} [object] The object to query keys on.
   * @returns {boolean} Returns `true` if `value` is a property name, else `false`.
   */
  function isKey$1(value, object) {
    if (isArray$1(value)) {
      return false;
    }
    var type = typeof value;
    if (type == 'number' || type == 'symbol' || type == 'boolean' ||
        value == null || isSymbol$2(value)) {
      return true;
    }
    return reIsPlainProp$1.test(value) || !reIsDeepProp$1.test(value) ||
      (object != null && value in Object(object));
  }

  /**
   * Checks if `value` is suitable for use as unique object key.
   *
   * @private
   * @param {*} value The value to check.
   * @returns {boolean} Returns `true` if `value` is suitable, else `false`.
   */
  function isKeyable$1(value) {
    var type = typeof value;
    return (type == 'string' || type == 'number' || type == 'symbol' || type == 'boolean')
      ? (value !== '__proto__')
      : (value === null);
  }

  /**
   * Checks if `func` has its source masked.
   *
   * @private
   * @param {Function} func The function to check.
   * @returns {boolean} Returns `true` if `func` is masked, else `false`.
   */
  function isMasked$1(func) {
    return !!maskSrcKey$1 && (maskSrcKey$1 in func);
  }

  /**
   * Converts `string` to a property path array.
   *
   * @private
   * @param {string} string The string to convert.
   * @returns {Array} Returns the property path array.
   */
  var stringToPath$1 = memoize$1(function(string) {
    string = toString$2(string);

    var result = [];
    if (reLeadingDot$1.test(string)) {
      result.push('');
    }
    string.replace(rePropName$1, function(match, number, quote, string) {
      result.push(quote ? string.replace(reEscapeChar$1, '$1') : (number || match));
    });
    return result;
  });

  /**
   * Converts `value` to a string key if it's not a string or symbol.
   *
   * @private
   * @param {*} value The value to inspect.
   * @returns {string|symbol} Returns the key.
   */
  function toKey$1(value) {
    if (typeof value == 'string' || isSymbol$2(value)) {
      return value;
    }
    var result = (value + '');
    return (result == '0' && (1 / value) == -INFINITY$2) ? '-0' : result;
  }

  /**
   * Converts `func` to its source code.
   *
   * @private
   * @param {Function} func The function to process.
   * @returns {string} Returns the source code.
   */
  function toSource$1(func) {
    if (func != null) {
      try {
        return funcToString$1.call(func);
      } catch (e) {}
      try {
        return (func + '');
      } catch (e) {}
    }
    return '';
  }

  /**
   * Creates a function that memoizes the result of `func`. If `resolver` is
   * provided, it determines the cache key for storing the result based on the
   * arguments provided to the memoized function. By default, the first argument
   * provided to the memoized function is used as the map cache key. The `func`
   * is invoked with the `this` binding of the memoized function.
   *
   * **Note:** The cache is exposed as the `cache` property on the memoized
   * function. Its creation may be customized by replacing the `_.memoize.Cache`
   * constructor with one whose instances implement the
   * [`Map`](http://ecma-international.org/ecma-262/7.0/#sec-properties-of-the-map-prototype-object)
   * method interface of `delete`, `get`, `has`, and `set`.
   *
   * @static
   * @memberOf _
   * @since 0.1.0
   * @category Function
   * @param {Function} func The function to have its output memoized.
   * @param {Function} [resolver] The function to resolve the cache key.
   * @returns {Function} Returns the new memoized function.
   * @example
   *
   * var object = { 'a': 1, 'b': 2 };
   * var other = { 'c': 3, 'd': 4 };
   *
   * var values = _.memoize(_.values);
   * values(object);
   * // => [1, 2]
   *
   * values(other);
   * // => [3, 4]
   *
   * object.a = 2;
   * values(object);
   * // => [1, 2]
   *
   * // Modify the result cache.
   * values.cache.set(object, ['a', 'b']);
   * values(object);
   * // => ['a', 'b']
   *
   * // Replace `_.memoize.Cache`.
   * _.memoize.Cache = WeakMap;
   */
  function memoize$1(func, resolver) {
    if (typeof func != 'function' || (resolver && typeof resolver != 'function')) {
      throw new TypeError(FUNC_ERROR_TEXT$1);
    }
    var memoized = function() {
      var args = arguments,
          key = resolver ? resolver.apply(this, args) : args[0],
          cache = memoized.cache;

      if (cache.has(key)) {
        return cache.get(key);
      }
      var result = func.apply(this, args);
      memoized.cache = cache.set(key, result);
      return result;
    };
    memoized.cache = new (memoize$1.Cache || MapCache$1);
    return memoized;
  }

  // Assign cache to `_.memoize`.
  memoize$1.Cache = MapCache$1;

  /**
   * Performs a
   * [`SameValueZero`](http://ecma-international.org/ecma-262/7.0/#sec-samevaluezero)
   * comparison between two values to determine if they are equivalent.
   *
   * @static
   * @memberOf _
   * @since 4.0.0
   * @category Lang
   * @param {*} value The value to compare.
   * @param {*} other The other value to compare.
   * @returns {boolean} Returns `true` if the values are equivalent, else `false`.
   * @example
   *
   * var object = { 'a': 1 };
   * var other = { 'a': 1 };
   *
   * _.eq(object, object);
   * // => true
   *
   * _.eq(object, other);
   * // => false
   *
   * _.eq('a', 'a');
   * // => true
   *
   * _.eq('a', Object('a'));
   * // => false
   *
   * _.eq(NaN, NaN);
   * // => true
   */
  function eq$1(value, other) {
    return value === other || (value !== value && other !== other);
  }

  /**
   * Checks if `value` is classified as an `Array` object.
   *
   * @static
   * @memberOf _
   * @since 0.1.0
   * @category Lang
   * @param {*} value The value to check.
   * @returns {boolean} Returns `true` if `value` is an array, else `false`.
   * @example
   *
   * _.isArray([1, 2, 3]);
   * // => true
   *
   * _.isArray(document.body.children);
   * // => false
   *
   * _.isArray('abc');
   * // => false
   *
   * _.isArray(_.noop);
   * // => false
   */
  var isArray$1 = Array.isArray;

  /**
   * Checks if `value` is classified as a `Function` object.
   *
   * @static
   * @memberOf _
   * @since 0.1.0
   * @category Lang
   * @param {*} value The value to check.
   * @returns {boolean} Returns `true` if `value` is a function, else `false`.
   * @example
   *
   * _.isFunction(_);
   * // => true
   *
   * _.isFunction(/abc/);
   * // => false
   */
  function isFunction$1(value) {
    // The use of `Object#toString` avoids issues with the `typeof` operator
    // in Safari 8-9 which returns 'object' for typed array and other constructors.
    var tag = isObject$1(value) ? objectToString$2.call(value) : '';
    return tag == funcTag$1 || tag == genTag$1;
  }

  /**
   * Checks if `value` is the
   * [language type](http://www.ecma-international.org/ecma-262/7.0/#sec-ecmascript-language-types)
   * of `Object`. (e.g. arrays, functions, objects, regexes, `new Number(0)`, and `new String('')`)
   *
   * @static
   * @memberOf _
   * @since 0.1.0
   * @category Lang
   * @param {*} value The value to check.
   * @returns {boolean} Returns `true` if `value` is an object, else `false`.
   * @example
   *
   * _.isObject({});
   * // => true
   *
   * _.isObject([1, 2, 3]);
   * // => true
   *
   * _.isObject(_.noop);
   * // => true
   *
   * _.isObject(null);
   * // => false
   */
  function isObject$1(value) {
    var type = typeof value;
    return !!value && (type == 'object' || type == 'function');
  }

  /**
   * Checks if `value` is object-like. A value is object-like if it's not `null`
   * and has a `typeof` result of "object".
   *
   * @static
   * @memberOf _
   * @since 4.0.0
   * @category Lang
   * @param {*} value The value to check.
   * @returns {boolean} Returns `true` if `value` is object-like, else `false`.
   * @example
   *
   * _.isObjectLike({});
   * // => true
   *
   * _.isObjectLike([1, 2, 3]);
   * // => true
   *
   * _.isObjectLike(_.noop);
   * // => false
   *
   * _.isObjectLike(null);
   * // => false
   */
  function isObjectLike$2(value) {
    return !!value && typeof value == 'object';
  }

  /**
   * Checks if `value` is classified as a `Symbol` primitive or object.
   *
   * @static
   * @memberOf _
   * @since 4.0.0
   * @category Lang
   * @param {*} value The value to check.
   * @returns {boolean} Returns `true` if `value` is a symbol, else `false`.
   * @example
   *
   * _.isSymbol(Symbol.iterator);
   * // => true
   *
   * _.isSymbol('abc');
   * // => false
   */
  function isSymbol$2(value) {
    return typeof value == 'symbol' ||
      (isObjectLike$2(value) && objectToString$2.call(value) == symbolTag$2);
  }

  /**
   * Converts `value` to a string. An empty string is returned for `null`
   * and `undefined` values. The sign of `-0` is preserved.
   *
   * @static
   * @memberOf _
   * @since 4.0.0
   * @category Lang
   * @param {*} value The value to process.
   * @returns {string} Returns the string.
   * @example
   *
   * _.toString(null);
   * // => ''
   *
   * _.toString(-0);
   * // => '-0'
   *
   * _.toString([1, 2, 3]);
   * // => '1,2,3'
   */
  function toString$2(value) {
    return value == null ? '' : baseToString$2(value);
  }

  /**
   * Sets the value at `path` of `object`. If a portion of `path` doesn't exist,
   * it's created. Arrays are created for missing index properties while objects
   * are created for all other missing properties. Use `_.setWith` to customize
   * `path` creation.
   *
   * **Note:** This method mutates `object`.
   *
   * @static
   * @memberOf _
   * @since 3.7.0
   * @category Object
   * @param {Object} object The object to modify.
   * @param {Array|string} path The path of the property to set.
   * @param {*} value The value to set.
   * @returns {Object} Returns `object`.
   * @example
   *
   * var object = { 'a': [{ 'b': { 'c': 3 } }] };
   *
   * _.set(object, 'a[0].b.c', 4);
   * console.log(object.a[0].b.c);
   * // => 4
   *
   * _.set(object, ['x', '0', 'y', 'z'], 5);
   * console.log(object.x[0].y.z);
   * // => 5
   */
  function set(object, path$$1, value) {
    return object == null ? object : baseSet(object, path$$1, value);
  }

  var lodash_set = set;

  var json = createCommonjsModule(function (module, exports) {
  Object.defineProperty(exports, "__esModule", { value: true });




  exports.stringify = function (target, replacer, offset) {
      if (!target || typeof target === 'string')
          return target;
      try {
          // try regular stringify first as mentioned in this tip: https://github.com/davidmarkclements/fast-safe-stringify#protip
          return JSON.stringify(target, replacer, offset);
      }
      catch (_) {
          // @ts-ignore
          return fastSafeStringify.default(target, replacer, offset);
      }
  };
  exports.parse = function (target) {
      if (typeof target !== 'string')
          return target;
      return JSON.parse(target);
  };
  exports.decycle = function (target, replacer) {
      if (typeof target !== 'object')
          return target;
      fastSafeStringify.default.decycle(target, replacer);
      return target;
  };
  exports.getValue = function (obj, path$$1, defaultVal) {
      return lodash_get(obj, path$$1, defaultVal);
  };
  exports.setValue = function (obj, path$$1, value) {
      return lodash_set(obj, path$$1, value);
  };
  exports.startsWith = function (source, val) {
      if (source instanceof Array) {
          if (val instanceof Array) {
              if (val.length > source.length)
                  return false;
              for (var i in val) {
                  if (!val.hasOwnProperty(i))
                      continue;
                  var si = parseInt(source[i]);
                  var vi = parseInt(val[i]);
                  // support if numeric index is stringified in one but not the other
                  if (!isNaN(si) || !isNaN(vi)) {
                      if (si !== vi) {
                          return false;
                      }
                  }
                  else if (source[i] !== val[i]) {
                      return false;
                  }
              }
          }
      }
      else if (typeof source === 'string') {
          if (typeof val === 'string') {
              return source.startsWith(val);
          }
      }
      else {
          return false;
      }
      return true;
  };
  /**
   * Removes elems from target, matched in order, starting on the left
   * trimStart([1, 2, 3], [1, 2]) === [3]
   * trimStart([1, 2, 3], [999, 2]) === [1, 2, 3] since source[0] does not equal elems[0]
   */
  exports.trimStart = function (target, elems) {
      if (typeof target === 'string') {
          return lodash_trimstart(target, elems);
      }
      if (!elems || !elems.length || !(elems instanceof Array))
          return target;
      var toRemove = 0;
      for (var i in target) {
          if (!target.hasOwnProperty(i))
              continue;
          if (target[i] !== elems[i])
              break;
          toRemove++;
      }
      return target.slice(toRemove);
  };

  });

  unwrapExports(json);
  var json_1 = json.stringify;
  var json_2 = json.parse;
  var json_3 = json.decycle;
  var json_4 = json.getValue;
  var json_5 = json.setValue;
  var json_6 = json.startsWith;
  var json_7 = json.trimStart;

  var uri = createCommonjsModule(function (module, exports) {
  // adapted from https://github.com/Microsoft/vscode-uri
  Object.defineProperty(exports, "__esModule", { value: true });
  // @ts-ignore

  var isWindows;
  if (typeof process === 'object') {
      isWindows = process.platform === 'win32';
  }
  else if (typeof navigator === 'object') {
      var userAgent = navigator.userAgent;
      isWindows = userAgent.indexOf('Windows') >= 0;
  }
  function _encode(ch) {
      return ('%' +
          ch
              .charCodeAt(0)
              .toString(16)
              .toUpperCase());
  }
  // see https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/encodeURIComponent
  function encodeURIComponent2(str) {
      return encodeURIComponent(str).replace(/[!'()*]/g, _encode);
  }
  function encodeNoop(str) {
      return str.replace(/[#?]/, _encode);
  }
  /**
   * Uniform Resource Identifier (URI) http://tools.ietf.org/html/rfc3986.
   * This class is a simple parser which creates the basic component paths
   * (http://tools.ietf.org/html/rfc3986#section-3) with minimal validation
   * and encoding.
   *
   *       foo://example.com:8042/over/there?name=ferret#nose
   *       \_/   \______________/\_________/ \_________/ \__/
   *        |           |            |            |        |
   *     scheme     authority       path        query   fragment
   *        |   _____________________|__
   *       / \ /                        \
   *       urn:example:animal:ferret:nose
   *
   *
   */
  var URI = /** @class */ (function () {
      function URI() {
          this._scheme = URI._empty;
          this._authority = URI._empty;
          this._path = URI._empty;
          this._query = URI._empty;
          this._fragment = URI._empty;
          this._formatted = null;
          this._fsPath = null;
      }
      URI.isUri = function (thing) {
          if (thing instanceof URI) {
              return true;
          }
          if (!thing) {
              return false;
          }
          return (typeof thing.authority === 'string' &&
              typeof thing.fragment === 'string' &&
              typeof thing.path === 'string' &&
              typeof thing.query === 'string' &&
              typeof thing.scheme === 'string');
      };
      // ---- parse & validate ------------------------
      URI.parse = function (value, decode) {
          var ret = new URI();
          var data = URI._parseComponents(value);
          ret._scheme = data.scheme;
          ret._authority = decode ? decodeURIComponent(data.authority) : data.authority;
          ret._path = decode ? decodeURIComponent(data.path) : data.path;
          ret._query = decode ? decodeURIComponent(data.query) : data.query;
          ret._fragment = decode ? decodeURIComponent(data.fragment) : data.fragment;
          URI._validate(ret);
          return ret;
      };
      URI.file = function (path$$1) {
          var ret = new URI();
          ret._scheme = 'file';
          // normalize to fwd-slashes on windows,
          // on other systems bwd-slaches are valid
          // filename character, eg /f\oo/ba\r.txt
          if (isWindows) {
              path$$1 = path$$1.replace(/\\/g, URI._slash);
          }
          // check for authority as used in UNC shares
          // or use the path as given
          if (path$$1[0] === URI._slash && path$$1[0] === path$$1[1]) {
              var idx = path$$1.indexOf(URI._slash, 2);
              if (idx === -1) {
                  ret._authority = path$$1.substring(2);
              }
              else {
                  ret._authority = path$$1.substring(2, idx);
                  ret._path = path$$1.substring(idx);
              }
          }
          else {
              ret._path = path$$1;
          }
          ret._path = path.normalize(ret._path);
          // Ensure that path starts with a slash
          // or that it is at least a slash
          if (ret._path[0] !== URI._slash) {
              ret._path = URI._slash + ret._path;
          }
          URI._validate(ret);
          return ret;
      };
      URI.from = function (components) {
          return new URI().with(components);
      };
      URI.revive = function (data) {
          var result = new URI();
          result._scheme = data.scheme || URI._empty;
          result._authority = data.authority || URI._empty;
          result._path = data.path || URI._empty;
          result._query = data.query || URI._empty;
          result._fragment = data.fragment || URI._empty;
          result._fsPath = data.fsPath;
          result._formatted = data.external;
          URI._validate(result);
          return result;
      };
      URI._parseComponents = function (value) {
          var ret = {
              scheme: URI._empty,
              authority: URI._empty,
              path: URI._empty,
              query: URI._empty,
              fragment: URI._empty,
          };
          var match = URI._regexp.exec(value);
          if (match) {
              ret.scheme = match[2] || ret.scheme;
              ret.authority = match[4] || ret.authority;
              ret.path = match[5] || ret.path;
              ret.query = match[7] || ret.query;
              ret.fragment = match[9] || ret.fragment;
          }
          return ret;
      };
      URI._validate = function (ret) {
          // scheme, https://tools.ietf.org/html/rfc3986#section-3.1
          // ALPHA *( ALPHA / DIGIT / "+" / "-" / "." )
          if (ret.scheme && !URI._schemePattern.test(ret.scheme)) {
              throw new Error('[UriError]: Scheme contains illegal characters.');
          }
          // path, http://tools.ietf.org/html/rfc3986#section-3.3
          // If a URI contains an authority component, then the path component
          // must either be empty or begin with a slash ("/") character.  If a URI
          // does not contain an authority component, then the path cannot begin
          // with two slash characters ("//").
          if (ret.path) {
              if (ret.authority) {
                  if (!URI._singleSlashStart.test(ret.path)) {
                      throw new Error('[UriError]: If a URI contains an authority component, then the path component must either be empty or begin with a slash ("/") character');
                  }
              }
              else {
                  if (URI._doubleSlashStart.test(ret.path)) {
                      throw new Error('[UriError]: If a URI does not contain an authority component, then the path cannot begin with two slash characters ("//")');
                  }
              }
          }
      };
      URI._asFormatted = function (uri, skipEncoding) {
          var encoder = !skipEncoding ? encodeURIComponent2 : encodeNoop;
          var parts = [];
          var scheme = uri.scheme, query = uri.query, fragment = uri.fragment;
          var authority = uri.authority, path$$1 = uri.path;
          if (scheme) {
              parts.push(scheme, ':');
          }
          if (authority || scheme === 'file') {
              parts.push('//');
          }
          if (authority) {
              authority = authority.toLowerCase();
              var idx = authority.indexOf(':');
              if (idx === -1) {
                  parts.push(encoder(authority));
              }
              else {
                  parts.push(encoder(authority.substr(0, idx)), authority.substr(idx));
              }
          }
          if (path$$1) {
              // lower-case windows drive letters in /C:/fff or C:/fff
              var m = URI._upperCaseDrive.exec(path$$1);
              if (m) {
                  if (m[1]) {
                      path$$1 = '/' + m[2].toLowerCase() + path$$1.substr(3); // "/c:".length === 3
                  }
                  else {
                      path$$1 = m[2].toLowerCase() + path$$1.substr(2); // // "c:".length === 2
                  }
              }
              if (scheme === 'mailto') {
                  parts.push(path$$1);
              }
              else {
                  // encode every segement but not slashes
                  // make sure that # and ? are always encoded
                  // when occurring in paths - otherwise the result
                  // cannot be parsed back again
                  var lastIdx = 0;
                  while (true) {
                      var idx = path$$1.indexOf(URI._slash, lastIdx);
                      if (idx === -1) {
                          parts.push(encoder(path$$1.substring(lastIdx)));
                          break;
                      }
                      parts.push(encoder(path$$1.substring(lastIdx, idx)), URI._slash);
                      lastIdx = idx + 1;
                  }
              }
          }
          if (query) {
              parts.push('?', encoder(query));
          }
          if (fragment) {
              parts.push('#', encoder(fragment));
          }
          return parts.join(URI._empty);
      };
      Object.defineProperty(URI.prototype, "scheme", {
          /**
           * scheme is the 'http' part of 'http://www.msft.com/some/path?query#fragment'.
           * The part before the first colon.
           */
          get: function () {
              return this._scheme;
          },
          enumerable: true,
          configurable: true
      });
      Object.defineProperty(URI.prototype, "authority", {
          /**
           * authority is the 'www.msft.com' part of 'http://www.msft.com/some/path?query#fragment'.
           * The part between the first double slashes and the next slash.
           */
          get: function () {
              return this._authority;
          },
          enumerable: true,
          configurable: true
      });
      Object.defineProperty(URI.prototype, "path", {
          /**
           * path is the '/some/path' part of 'http://www.msft.com/some/path?query#fragment'.
           */
          get: function () {
              return this._path;
          },
          enumerable: true,
          configurable: true
      });
      Object.defineProperty(URI.prototype, "query", {
          /**
           * query is the 'query' part of 'http://www.msft.com/some/path?query#fragment'.
           */
          get: function () {
              return this._query;
          },
          enumerable: true,
          configurable: true
      });
      Object.defineProperty(URI.prototype, "fragment", {
          /**
           * fragment is the 'fragment' part of 'http://www.msft.com/some/path?query#fragment'.
           */
          get: function () {
              return this._fragment;
          },
          enumerable: true,
          configurable: true
      });
      Object.defineProperty(URI.prototype, "fsPath", {
          // ---- filesystem path -----------------------
          /**
           * Returns a string representing the corresponding file system path of this URI.
           * Will handle UNC paths and normalize windows drive letters to lower-case. Also
           * uses the platform specific path separator. Will *not* validate the path for
           * invalid characters and semantics. Will *not* look at the scheme of this URI.
           */
          get: function () {
              if (!this._fsPath) {
                  var value = void 0;
                  if (this._authority && this._path && this.scheme === 'file') {
                      // unc path: file://shares/c$/far/boo
                      value = "//" + this._authority + path.normalize(this._path);
                  }
                  else if (URI._driveLetterPath.test(this._path)) {
                      // windows drive letter: file:///c:/far/boo
                      value = this._path[1].toLowerCase() + path.normalize(this._path.substr(2));
                  }
                  else {
                      // other path
                      value = path.normalize(this._path);
                  }
                  if (isWindows) {
                      value = value.replace(/\//g, '\\');
                  }
                  this._fsPath = value;
              }
              return this._fsPath;
          },
          enumerable: true,
          configurable: true
      });
      Object.defineProperty(URI.prototype, "origin", {
          get: function () {
              if (this.scheme && this.authority) {
                  return this.scheme + "://" + this.authority;
              }
              return '';
          },
          enumerable: true,
          configurable: true
      });
      Object.defineProperty(URI.prototype, "location", {
          get: function () {
              var res = {
                  hash: '',
                  host: '',
                  hostname: '',
                  href: this.toURL(),
                  origin: this.origin,
                  pathname: '',
                  port: '',
                  protocol: '',
                  search: '',
              };
              if (this.scheme) {
                  res.protocol = this.scheme + ":";
              }
              if (this.authority) {
                  res.host = this.authority;
                  var parts = this.authority.split(':');
                  res.hostname = parts[0];
                  res.port = parts[1];
              }
              if (this.path) {
                  res.pathname = this.path;
              }
              if (this.query) {
                  res.search = "?" + this.query;
              }
              if (this.fragment) {
                  res.hash = "#" + this.fragment;
              }
              return res;
          },
          enumerable: true,
          configurable: true
      });
      URI.prototype.isAbsolute = function () {
          return this.scheme === 'mailto' || (this.scheme && this.authority) ? true : false;
      };
      URI.prototype.isFile = function (root) {
          if (root && this.scheme === '') {
              return root.scheme === '' || root.scheme === 'file';
          }
          return this.scheme === 'file';
      };
      URI.prototype.isJSONPointer = function () {
          return this.fragment !== '' && this.scheme === '' && this.authority === '' && this.path === '';
      };
      // ---- modify to new -------------------------
      URI.prototype.with = function (change) {
          if (!change) {
              return this;
          }
          var scheme = change.scheme, authority = change.authority, path$$1 = change.path, query = change.query, fragment = change.fragment;
          if (scheme === void 0) {
              scheme = this.scheme;
          }
          else if (scheme === null) {
              scheme = '';
          }
          if (authority === void 0) {
              authority = this.authority;
          }
          else if (authority === null) {
              authority = '';
          }
          if (path$$1 === void 0) {
              path$$1 = this.path;
          }
          else if (path$$1 === null) {
              path$$1 = '';
          }
          if (query === void 0) {
              query = this.query;
          }
          else if (query === null) {
              query = '';
          }
          if (fragment === void 0) {
              fragment = this.fragment;
          }
          else if (fragment === null) {
              fragment = '';
          }
          if (scheme === this.scheme &&
              authority === this.authority &&
              path$$1 === this.path &&
              query === this.query &&
              fragment === this.fragment) {
              return this;
          }
          var ret = new URI();
          ret._scheme = scheme;
          ret._authority = authority;
          ret._path = path$$1;
          ret._query = query;
          ret._fragment = fragment;
          if (ret.isFile()) {
              ret._path = path.normalize(ret._path);
          }
          URI._validate(ret);
          return ret;
      };
      // ---- printing/externalize ---------------------------
      /**
       *
       * @param skipEncoding Do not encode the result, default is `true`
       */
      URI.prototype.toString = function (skipEncoding) {
          if (skipEncoding === void 0) { skipEncoding = true; }
          if (!skipEncoding) {
              if (!this._formatted) {
                  this._formatted = URI._asFormatted(this, false);
              }
              return this._formatted;
          }
          else {
              // we don't cache that
              return URI._asFormatted(this, true);
          }
      };
      URI.prototype.toURL = function () {
          var url = '';
          if (this._scheme) {
              url += this._scheme + ":";
              if (this._authority || this._scheme === 'file') {
                  url += '//';
              }
          }
          if (this._authority) {
              url += this._authority;
          }
          if (this._path) {
              url += this._path;
          }
          if (this._query) {
              url += "?" + this._query;
          }
          if (this._fragment) {
              url += "#" + this._fragment;
          }
          return url;
      };
      URI.prototype.toJSON = function () {
          var res = {
              fsPath: this.fsPath,
              external: this.toString(true),
              $mid: 1,
          };
          if (this.path) {
              res.path = this.path;
          }
          if (this.scheme) {
              res.scheme = this.scheme;
          }
          if (this.authority) {
              res.authority = this.authority;
          }
          if (this.query) {
              res.query = this.query;
          }
          if (this.fragment) {
              res.fragment = this.fragment;
          }
          return res;
      };
      URI.prototype.toJSONPointer = function () {
          return this.fragment ? "#" + (this.fragment || '/') : '';
      };
      URI._schemePattern = /^\w[\w\d+.-]*$/;
      URI._singleSlashStart = /^\//;
      URI._doubleSlashStart = /^\/\//;
      URI._empty = '';
      URI._slash = '/';
      URI._regexp = /^(([^:/?#]+?):)?(\/\/([^/?#]*))?([^?#]*)(\?([^#]*))?(#(.*))?/;
      URI._driveLetterPath = /^\/[a-zA-z]:/;
      URI._upperCaseDrive = /^(\/)?([A-Z]:)/;
      return URI;
  }());
  exports.URI = URI;

  });

  unwrapExports(uri);
  var uri_1 = uri.URI;

  function generatePatches(state, basepath, patches, inversePatches, baseValue, resultValue) {
      if (patches) if (Array.isArray(baseValue)) generateArrayPatches(state, basepath, patches, inversePatches, baseValue, resultValue);else generateObjectPatches(state, basepath, patches, inversePatches, baseValue, resultValue);
  }

  function generateArrayPatches(state, basepath, patches, inversePatches, baseValue, resultValue) {
      var shared = Math.min(baseValue.length, resultValue.length);
      for (var i = 0; i < shared; i++) {
          if (state.assigned[i] && baseValue[i] !== resultValue[i]) {
              var path$$1 = basepath.concat(i);
              patches.push({ op: "replace", path: path$$1, value: resultValue[i] });
              inversePatches.push({ op: "replace", path: path$$1, value: baseValue[i] });
          }
      }
      if (shared < resultValue.length) {
          // stuff was added
          for (var _i = shared; _i < resultValue.length; _i++) {
              var _path = basepath.concat(_i);
              patches.push({ op: "add", path: _path, value: resultValue[_i] });
          }
          inversePatches.push({
              op: "replace",
              path: basepath.concat("length"),
              value: baseValue.length
          });
      } else if (shared < baseValue.length) {
          // stuff was removed
          patches.push({
              op: "replace",
              path: basepath.concat("length"),
              value: resultValue.length
          });
          for (var _i2 = shared; _i2 < baseValue.length; _i2++) {
              var _path2 = basepath.concat(_i2);
              inversePatches.push({ op: "add", path: _path2, value: baseValue[_i2] });
          }
      }
  }

  function generateObjectPatches(state, basepath, patches, inversePatches, baseValue, resultValue) {
      each(state.assigned, function (key, assignedValue) {
          var origValue = baseValue[key];
          var value = resultValue[key];
          var op = !assignedValue ? "remove" : key in baseValue ? "replace" : "add";
          if (origValue === baseValue && op === "replace") return;
          var path$$1 = basepath.concat(key);
          patches.push(op === "remove" ? { op: op, path: path$$1 } : { op: op, path: path$$1, value: value });
          inversePatches.push(op === "add" ? { op: "remove", path: path$$1 } : op === "remove" ? { op: "add", path: path$$1, value: origValue } : { op: "replace", path: path$$1, value: origValue });
      });
  }

  function applyPatches(draft, patches) {
      var _loop = function _loop(i) {
          var patch = patches[i];
          if (patch.path.length === 0 && patch.op === "replace") {
              draft = patch.value;
          } else {
              var path$$1 = patch.path.slice();
              var key = path$$1.pop();
              var base = path$$1.reduce(function (current, part) {
                  if (!current) throw new Error("Cannot apply patch, path doesn't resolve: " + patch.path.join("/"));
                  return current[part];
              }, draft);
              if (!base) throw new Error("Cannot apply patch, path doesn't resolve: " + patch.path.join("/"));
              switch (patch.op) {
                  case "replace":
                  case "add":
                      // TODO: add support is not extensive, it does not support insertion or `-` atm!
                      base[key] = patch.value;
                      break;
                  case "remove":
                      if (Array.isArray(base)) {
                          if (key === base.length - 1) base.length -= 1;else throw new Error("Remove can only remove the last key of an array, index: " + key + ", length: " + base.length);
                      } else delete base[key];
                      break;
                  default:
                      throw new Error("Unsupported patch operation: " + patch.op);
              }
          }
      };

      for (var i = 0; i < patches.length; i++) {
          _loop(i);
      }
      return draft;
  }

  var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) {
    return typeof obj;
  } : function (obj) {
    return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj;
  };



















  var defineProperty = function (obj, key, value) {
    if (key in obj) {
      Object.defineProperty(obj, key, {
        value: value,
        enumerable: true,
        configurable: true,
        writable: true
      });
    } else {
      obj[key] = value;
    }

    return obj;
  };

  var NOTHING = typeof Symbol !== "undefined" ? Symbol("immer-nothing") : defineProperty({}, "immer-nothing", true);

  var PROXY_STATE = typeof Symbol !== "undefined" ? Symbol("immer-proxy-state") : "__$immer_state";

  var RETURNED_AND_MODIFIED_ERROR = "An immer producer returned a new value *and* modified its draft. Either return a new value *or* modify the draft.";

  function verifyMinified() {}

  var inProduction = typeof process !== "undefined" && process.env.NODE_ENV === "production" || verifyMinified.name !== "verifyMinified";

  var autoFreeze = !inProduction;
  var useProxies = typeof Proxy !== "undefined";

  function getUseProxies() {
      return useProxies;
  }

  function isProxy(value) {
      return !!value && !!value[PROXY_STATE];
  }

  function isProxyable(value) {
      if (!value) return false;
      if ((typeof value === "undefined" ? "undefined" : _typeof(value)) !== "object") return false;
      if (Array.isArray(value)) return true;
      var proto = Object.getPrototypeOf(value);
      return proto === null || proto === Object.prototype;
  }

  function freeze(value) {
      if (autoFreeze) {
          Object.freeze(value);
      }
      return value;
  }

  var assign = Object.assign || function assign(target, value) {
      for (var key in value) {
          if (has(value, key)) {
              target[key] = value[key];
          }
      }
      return target;
  };

  function shallowCopy(value) {
      if (Array.isArray(value)) return value.slice();
      var target = value.__proto__ === undefined ? Object.create(null) : {};
      return assign(target, value);
  }

  function each(value, cb) {
      if (Array.isArray(value)) {
          for (var i = 0; i < value.length; i++) {
              cb(i, value[i]);
          }
      } else {
          for (var key in value) {
              cb(key, value[key]);
          }
      }
  }

  function has(thing, prop) {
      return Object.prototype.hasOwnProperty.call(thing, prop);
  }

  // given a base object, returns it if unmodified, or return the changed cloned if modified
  function finalize(base, path$$1, patches, inversePatches) {
      if (isProxy(base)) {
          var state = base[PROXY_STATE];
          if (state.modified === true) {
              if (state.finalized === true) return state.copy;
              state.finalized = true;
              var result = finalizeObject(useProxies ? state.copy : state.copy = shallowCopy(base), state, path$$1, patches, inversePatches);
              generatePatches(state, path$$1, patches, inversePatches, state.base, result);
              return result;
          } else {
              return state.base;
          }
      }
      finalizeNonProxiedObject(base);
      return base;
  }

  function finalizeObject(copy, state, path$$1, patches, inversePatches) {
      var base = state.base;
      each(copy, function (prop, value) {
          if (value !== base[prop]) {
              // if there was an assignment on this property, we don't need to generate
              // patches for the subtree
              var _generatePatches = patches && !has(state.assigned, prop);
              copy[prop] = finalize(value, _generatePatches && path$$1.concat(prop), _generatePatches && patches, inversePatches);
          }
      });
      return freeze(copy);
  }

  function finalizeNonProxiedObject(parent) {
      // If finalize is called on an object that was not a proxy, it means that it is an object that was not there in the original
      // tree and it could contain proxies at arbitrarily places. Let's find and finalize them as well
      if (!isProxyable(parent)) return;
      if (Object.isFrozen(parent)) return;
      each(parent, function (i, child) {
          if (isProxy(child)) {
              parent[i] = finalize(child);
          } else finalizeNonProxiedObject(child);
      });
      // always freeze completely new data
      freeze(parent);
  }



  function is(x, y) {
      // From: https://github.com/facebook/fbjs/blob/c69904a511b900266935168223063dd8772dfc40/packages/fbjs/src/core/shallowEqual.js
      if (x === y) {
          return x !== 0 || 1 / x === 1 / y;
      } else {
          return x !== x && y !== y;
      }
  }

  // @ts-check

  var proxies = null;

  var objectTraps = {
      get: get$1,
      has: function has$$1(target, prop) {
          return prop in source(target);
      },
      ownKeys: function ownKeys(target) {
          return Reflect.ownKeys(source(target));
      },

      set: set$1,
      deleteProperty: deleteProperty,
      getOwnPropertyDescriptor: getOwnPropertyDescriptor,
      defineProperty: defineProperty$1,
      setPrototypeOf: function setPrototypeOf() {
          throw new Error("Immer does not support `setPrototypeOf()`.");
      }
  };

  var arrayTraps = {};
  each(objectTraps, function (key, fn) {
      arrayTraps[key] = function () {
          arguments[0] = arguments[0][0];
          return fn.apply(this, arguments);
      };
  });
  arrayTraps.deleteProperty = function (state, prop) {
      if (isNaN(parseInt(prop))) throw new Error("Immer does not support deleting properties from arrays: " + prop);
      return objectTraps.deleteProperty.call(this, state[0], prop);
  };
  arrayTraps.set = function (state, prop, value) {
      if (prop !== "length" && isNaN(parseInt(prop))) throw new Error("Immer does not support setting non-numeric properties on arrays: " + prop);
      return objectTraps.set.call(this, state[0], prop, value);
  };

  function createState(parent, base) {
      return {
          modified: false, // this tree is modified (either this object or one of it's children)
          assigned: {}, // true: value was assigned to these props, false: was removed
          finalized: false,
          parent: parent,
          base: base,
          copy: undefined,
          proxies: {}
      };
  }

  function source(state) {
      return state.modified === true ? state.copy : state.base;
  }

  function get$1(state, prop) {
      if (prop === PROXY_STATE) return state;
      if (state.modified) {
          var value = state.copy[prop];
          if (value === state.base[prop] && isProxyable(value))
              // only create proxy if it is not yet a proxy, and not a new object
              // (new objects don't need proxying, they will be processed in finalize anyway)
              return state.copy[prop] = createProxy(state, value);
          return value;
      } else {
          if (has(state.proxies, prop)) return state.proxies[prop];
          var _value = state.base[prop];
          if (!isProxy(_value) && isProxyable(_value)) return state.proxies[prop] = createProxy(state, _value);
          return _value;
      }
  }

  function set$1(state, prop, value) {
      // TODO: optimize
      state.assigned[prop] = true;
      if (!state.modified) {
          if (prop in state.base && is(state.base[prop], value) || has(state.proxies, prop) && state.proxies[prop] === value) return true;
          markChanged(state);
      }
      state.copy[prop] = value;
      return true;
  }

  function deleteProperty(state, prop) {
      state.assigned[prop] = false;
      markChanged(state);
      delete state.copy[prop];
      return true;
  }

  function getOwnPropertyDescriptor(state, prop) {
      var owner = state.modified ? state.copy : has(state.proxies, prop) ? state.proxies : state.base;
      var descriptor = Reflect.getOwnPropertyDescriptor(owner, prop);
      if (descriptor && !(Array.isArray(owner) && prop === "length")) descriptor.configurable = true;
      return descriptor;
  }

  function defineProperty$1() {
      throw new Error("Immer does not support defining properties on draft objects.");
  }

  function markChanged(state) {
      if (!state.modified) {
          state.modified = true;
          state.copy = shallowCopy(state.base);
          // copy the proxies over the base-copy
          Object.assign(state.copy, state.proxies); // yup that works for arrays as well
          if (state.parent) markChanged(state.parent);
      }
  }

  // creates a proxy for plain objects / arrays
  function createProxy(parentState, base, key) {
      if (isProxy(base)) throw new Error("Immer bug. Plz report.");
      var state = createState(parentState, base, key);
      var proxy = Array.isArray(base) ? Proxy.revocable([state], arrayTraps) : Proxy.revocable(state, objectTraps);
      proxies.push(proxy);
      return proxy.proxy;
  }

  function produceProxy(baseState, producer, patchListener) {
      if (isProxy(baseState)) {
          // See #100, don't nest producers
          var returnValue = producer.call(baseState, baseState);
          return returnValue === undefined ? baseState : returnValue;
      }
      var previousProxies = proxies;
      proxies = [];
      var patches = patchListener && [];
      var inversePatches = patchListener && [];
      try {
          // create proxy for root
          var rootProxy = createProxy(undefined, baseState);
          // execute the thunk
          var _returnValue = producer.call(rootProxy, rootProxy);
          // and finalize the modified proxy
          var result = void 0;
          // check whether the draft was modified and/or a value was returned
          if (_returnValue !== undefined && _returnValue !== rootProxy) {
              // something was returned, and it wasn't the proxy itself
              if (rootProxy[PROXY_STATE].modified) throw new Error(RETURNED_AND_MODIFIED_ERROR);

              // See #117
              // Should we just throw when returning a proxy which is not the root, but a subset of the original state?
              // Looks like a wrongly modeled reducer
              result = finalize(_returnValue);
              if (patches) {
                  patches.push({ op: "replace", path: [], value: result });
                  inversePatches.push({ op: "replace", path: [], value: baseState });
              }
          } else {
              result = finalize(rootProxy, [], patches, inversePatches);
          }
          // revoke all proxies
          each(proxies, function (_, p) {
              return p.revoke();
          });
          patchListener && patchListener(patches, inversePatches);
          return result;
      } finally {
          proxies = previousProxies;
      }
  }

  // @ts-check

  var descriptors = {};
  var states = null;

  function createState$1(parent, proxy, base) {
      return {
          modified: false,
          assigned: {}, // true: value was assigned to these props, false: was removed
          hasCopy: false,
          parent: parent,
          base: base,
          proxy: proxy,
          copy: undefined,
          finished: false,
          finalizing: false,
          finalized: false
      };
  }

  function source$1(state) {
      return state.hasCopy ? state.copy : state.base;
  }

  function _get(state, prop) {
      assertUnfinished(state);
      var value = source$1(state)[prop];
      if (!state.finalizing && value === state.base[prop] && isProxyable(value)) {
          // only create a proxy if the value is proxyable, and the value was in the base state
          // if it wasn't in the base state, the object is already modified and we will process it in finalize
          prepareCopy(state);
          return state.copy[prop] = createProxy$1(state, value);
      }
      return value;
  }

  function _set(state, prop, value) {
      assertUnfinished(state);
      state.assigned[prop] = true; // optimization; skip this if there is no listener
      if (!state.modified) {
          if (is(source$1(state)[prop], value)) return;
          markChanged$1(state);
          prepareCopy(state);
      }
      state.copy[prop] = value;
  }

  function markChanged$1(state) {
      if (!state.modified) {
          state.modified = true;
          if (state.parent) markChanged$1(state.parent);
      }
  }

  function prepareCopy(state) {
      if (state.hasCopy) return;
      state.hasCopy = true;
      state.copy = shallowCopy(state.base);
  }

  // creates a proxy for plain objects / arrays
  function createProxy$1(parent, base) {
      var proxy = shallowCopy(base);
      each(base, function (i) {
          Object.defineProperty(proxy, "" + i, createPropertyProxy("" + i));
      });
      var state = createState$1(parent, proxy, base);
      createHiddenProperty(proxy, PROXY_STATE, state);
      states.push(state);
      return proxy;
  }

  function createPropertyProxy(prop) {
      return descriptors[prop] || (descriptors[prop] = {
          configurable: true,
          enumerable: true,
          get: function get$$1() {
              return _get(this[PROXY_STATE], prop);
          },
          set: function set$$1(value) {
              _set(this[PROXY_STATE], prop, value);
          }
      });
  }

  function assertUnfinished(state) {
      if (state.finished === true) throw new Error("Cannot use a proxy that has been revoked. Did you pass an object from inside an immer function to an async process? " + JSON.stringify(state.copy || state.base));
  }

  // this sounds very expensive, but actually it is not that expensive in practice
  // as it will only visit proxies, and only do key-based change detection for objects for
  // which it is not already know that they are changed (that is, only object for which no known key was changed)
  function markChangesSweep() {
      // intentionally we process the proxies in reverse order;
      // ideally we start by processing leafs in the tree, because if a child has changed, we don't have to check the parent anymore
      // reverse order of proxy creation approximates this
      for (var i = states.length - 1; i >= 0; i--) {
          var state = states[i];
          if (state.modified === false) {
              if (Array.isArray(state.base)) {
                  if (hasArrayChanges(state)) markChanged$1(state);
              } else if (hasObjectChanges(state)) markChanged$1(state);
          }
      }
  }

  function markChangesRecursively(object) {
      if (!object || (typeof object === "undefined" ? "undefined" : _typeof(object)) !== "object") return;
      var state = object[PROXY_STATE];
      if (!state) return;
      var proxy = state.proxy,
          base = state.base;

      if (Array.isArray(object)) {
          if (hasArrayChanges(state)) {
              markChanged$1(state);
              state.assigned.length = true;
              if (proxy.length < base.length) for (var i = proxy.length; i < base.length; i++) {
                  state.assigned[i] = false;
              } else for (var _i = base.length; _i < proxy.length; _i++) {
                  state.assigned[_i] = true;
              }each(proxy, function (index, child) {
                  if (!state.assigned[index]) markChangesRecursively(child);
              });
          }
      } else {
          var _diffKeys = diffKeys(base, proxy),
              added = _diffKeys.added,
              removed = _diffKeys.removed;

          if (added.length > 0 || removed.length > 0) markChanged$1(state);
          each(added, function (_, key) {
              state.assigned[key] = true;
          });
          each(removed, function (_, key) {
              state.assigned[key] = false;
          });
          each(proxy, function (key, child) {
              if (!state.assigned[key]) markChangesRecursively(child);
          });
      }
  }

  function diffKeys(from, to) {
      // TODO: optimize
      var a = Object.keys(from);
      var b = Object.keys(to);
      return {
          added: b.filter(function (key) {
              return a.indexOf(key) === -1;
          }),
          removed: a.filter(function (key) {
              return b.indexOf(key) === -1;
          })
      };
  }

  function hasObjectChanges(state) {
      var baseKeys = Object.keys(state.base);
      var keys = Object.keys(state.proxy);
      return !shallowEqual(baseKeys, keys);
  }

  function hasArrayChanges(state) {
      var proxy = state.proxy;

      if (proxy.length !== state.base.length) return true;
      // See #116
      // If we first shorten the length, our array interceptors will be removed.
      // If after that new items are added, result in the same original length,
      // those last items will have no intercepting property.
      // So if there is no own descriptor on the last position, we know that items were removed and added
      // N.B.: splice, unshift, etc only shift values around, but not prop descriptors, so we only have to check
      // the last one
      var descriptor = Object.getOwnPropertyDescriptor(proxy, proxy.length - 1);
      // descriptor can be null, but only for newly created sparse arrays, eg. new Array(10)
      if (descriptor && !descriptor.get) return true;
      // For all other cases, we don't have to compare, as they would have been picked up by the index setters
      return false;
  }

  function produceEs5(baseState, producer, patchListener) {
      if (isProxy(baseState)) {
          // See #100, don't nest producers
          var returnValue = producer.call(baseState, baseState);
          return returnValue === undefined ? baseState : returnValue;
      }
      var prevStates = states;
      states = [];
      var patches = patchListener && [];
      var inversePatches = patchListener && [];
      try {
          // create proxy for root
          var rootProxy = createProxy$1(undefined, baseState);
          // execute the thunk
          var _returnValue = producer.call(rootProxy, rootProxy);
          // and finalize the modified proxy
          each(states, function (_, state) {
              state.finalizing = true;
          });
          var result = void 0;
          // check whether the draft was modified and/or a value was returned
          if (_returnValue !== undefined && _returnValue !== rootProxy) {
              // something was returned, and it wasn't the proxy itself
              if (rootProxy[PROXY_STATE].modified) throw new Error(RETURNED_AND_MODIFIED_ERROR);
              result = finalize(_returnValue);
              if (patches) {
                  patches.push({ op: "replace", path: [], value: result });
                  inversePatches.push({ op: "replace", path: [], value: baseState });
              }
          } else {
              if (patchListener) markChangesRecursively(rootProxy);
              markChangesSweep(); // this one is more efficient if we don't need to know which attributes have changed
              result = finalize(rootProxy, [], patches, inversePatches);
          }
          // make sure all proxies become unusable
          each(states, function (_, state) {
              state.finished = true;
          });
          patchListener && patchListener(patches, inversePatches);
          return result;
      } finally {
          states = prevStates;
      }
  }

  function shallowEqual(objA, objB) {
      //From: https://github.com/facebook/fbjs/blob/c69904a511b900266935168223063dd8772dfc40/packages/fbjs/src/core/shallowEqual.js
      if (is(objA, objB)) return true;
      if ((typeof objA === "undefined" ? "undefined" : _typeof(objA)) !== "object" || objA === null || (typeof objB === "undefined" ? "undefined" : _typeof(objB)) !== "object" || objB === null) {
          return false;
      }
      var keysA = Object.keys(objA);
      var keysB = Object.keys(objB);
      if (keysA.length !== keysB.length) return false;
      for (var i = 0; i < keysA.length; i++) {
          if (!hasOwnProperty.call(objB, keysA[i]) || !is(objA[keysA[i]], objB[keysA[i]])) {
              return false;
          }
      }
      return true;
  }

  function createHiddenProperty(target, prop, value) {
      Object.defineProperty(target, prop, {
          value: value,
          enumerable: false,
          writable: true
      });
  }

  /**
   * produce takes a state, and runs a function against it.
   * That function can freely mutate the state, as it will create copies-on-write.
   * This means that the original state will stay unchanged, and once the function finishes, the modified state is returned
   *
   * @export
   * @param {any} baseState - the state to start with
   * @param {Function} producer - function that receives a proxy of the base state as first argument and which can be freely modified
   * @param {Function} patchListener - optional function that will be called with all the patches produced here
   * @returns {any} a new state, or the base state if nothing was modified
   */
  function produce(baseState, producer, patchListener) {
      // prettier-ignore
      if (arguments.length < 1 || arguments.length > 3) throw new Error("produce expects 1 to 3 arguments, got " + arguments.length);

      // curried invocation
      if (typeof baseState === "function") {
          // prettier-ignore
          if (typeof producer === "function") throw new Error("if first argument is a function (curried invocation), the second argument to produce cannot be a function");

          var initialState = producer;
          var recipe = baseState;

          return function () {
              var args = arguments;

              var currentState = args[0] === undefined && initialState !== undefined ? initialState : args[0];

              return produce(currentState, function (draft) {
                  args[0] = draft; // blegh!
                  return recipe.apply(draft, args);
              });
          };
      }

      // prettier-ignore
      {
          if (typeof producer !== "function") throw new Error("if first argument is not a function, the second argument to produce should be a function");
          if (patchListener !== undefined && typeof patchListener !== "function") throw new Error("the third argument of a producer should not be set or a function");
      }

      // if state is a primitive, don't bother proxying at all
      if ((typeof baseState === "undefined" ? "undefined" : _typeof(baseState)) !== "object" || baseState === null) {
          var returnValue = producer(baseState);
          return returnValue === undefined ? baseState : normalizeResult(returnValue);
      }

      if (!isProxyable(baseState)) throw new Error("the first argument to an immer producer should be a primitive, plain object or array, got " + (typeof baseState === "undefined" ? "undefined" : _typeof(baseState)) + ": \"" + baseState + "\"");
      return normalizeResult(getUseProxies() ? produceProxy(baseState, producer, patchListener) : produceEs5(baseState, producer, patchListener));
  }

  function normalizeResult(result) {
      return result === NOTHING ? undefined : result;
  }

  var applyPatches$1 = produce(applyPatches);

  var dep_graph = createCommonjsModule(function (module, exports) {
  /**
   * A simple dependency graph
   */

  /**
   * Helper for creating a Depth-First-Search on
   * a set of edges.
   *
   * Detects cycles and throws an Error if one is detected.
   *
   * @param edges The set of edges to DFS through
   * @param leavesOnly Whether to only return "leaf" nodes (ones who have no edges)
   * @param result An array in which the results will be populated
   * @param circular A boolean to allow circular dependencies
   */
  function createDFS(edges, leavesOnly, result, circular) {
    var currentPath = [];
    var visited = {};
    return function DFS(currentNode) {
      visited[currentNode] = true;
      currentPath.push(currentNode);
      edges[currentNode].forEach(function (node) {
        if (!visited[node]) {
          DFS(node);
        } else if (currentPath.indexOf(node) >= 0) {
          currentPath.push(node);
          if (!circular) {
            throw new Error('Dependency Cycle Found: ' + currentPath.join(' -> '));
          }
        }
      });
      currentPath.pop();
      if ((!leavesOnly || edges[currentNode].length === 0) && result.indexOf(currentNode) === -1) {
        result.push(currentNode);
      }
    };
  }

  /**
   * Simple Dependency Graph
   */
  var DepGraph = exports.DepGraph = function DepGraph(opts) {
    this.nodes = {}; // Node -> Node/Data (treated like a Set)
    this.outgoingEdges = {}; // Node -> [Dependency Node]
    this.incomingEdges = {}; // Node -> [Dependant Node]
    this.circular = opts && !!opts.circular; // Allows circular deps
  };
  DepGraph.prototype = {
    /**
     * The number of nodes in the graph.
     */
    size:function () {
      return Object.keys(this.nodes).length;
    },
    /**
     * Add a node to the dependency graph. If a node already exists, this method will do nothing.
     */
    addNode:function (node, data) {
      if (!this.hasNode(node)) {
        // Checking the arguments length allows the user to add a node with undefined data
        if (arguments.length === 2) {
          this.nodes[node] = data;
        } else {
          this.nodes[node] = node;
        }
        this.outgoingEdges[node] = [];
        this.incomingEdges[node] = [];
      }
    },
    /**
     * Remove a node from the dependency graph. If a node does not exist, this method will do nothing.
     */
    removeNode:function (node) {
      if (this.hasNode(node)) {
        delete this.nodes[node];
        delete this.outgoingEdges[node];
        delete this.incomingEdges[node];
        [this.incomingEdges, this.outgoingEdges].forEach(function (edgeList) {
          Object.keys(edgeList).forEach(function (key) {
            var idx = edgeList[key].indexOf(node);
            if (idx >= 0) {
              edgeList[key].splice(idx, 1);
            }
          }, this);
        });
      }
    },
    /**
     * Check if a node exists in the graph
     */
    hasNode:function (node) {
      return this.nodes.hasOwnProperty(node);
    },
    /**
     * Get the data associated with a node name
     */
    getNodeData:function (node) {
      if (this.hasNode(node)) {
        return this.nodes[node];
      } else {
        throw new Error('Node does not exist: ' + node);
      }
    },
    /**
     * Set the associated data for a given node name. If the node does not exist, this method will throw an error
     */
    setNodeData:function (node, data) {
      if (this.hasNode(node)) {
        this.nodes[node] = data;
      } else {
        throw new Error('Node does not exist: ' + node);
      }
    },
    /**
     * Add a dependency between two nodes. If either of the nodes does not exist,
     * an Error will be thrown.
     */
    addDependency:function (from, to) {
      if (!this.hasNode(from)) {
        throw new Error('Node does not exist: ' + from);
      }
      if (!this.hasNode(to)) {
        throw new Error('Node does not exist: ' + to);
      }
      if (this.outgoingEdges[from].indexOf(to) === -1) {
        this.outgoingEdges[from].push(to);
      }
      if (this.incomingEdges[to].indexOf(from) === -1) {
        this.incomingEdges[to].push(from);
      }
      return true;
    },
    /**
     * Remove a dependency between two nodes.
     */
    removeDependency:function (from, to) {
      var idx;
      if (this.hasNode(from)) {
        idx = this.outgoingEdges[from].indexOf(to);
        if (idx >= 0) {
          this.outgoingEdges[from].splice(idx, 1);
        }
      }

      if (this.hasNode(to)) {
        idx = this.incomingEdges[to].indexOf(from);
        if (idx >= 0) {
          this.incomingEdges[to].splice(idx, 1);
        }
      }
    },
    /**
     * Return a clone of the dependency graph. If any custom data is attached
     * to the nodes, it will only be shallow copied.
     */
    clone:function () {
      var source = this;
      var result = new DepGraph();
      var keys = Object.keys(source.nodes);
      keys.forEach(function (n) {
        result.nodes[n] = source.nodes[n];
        result.outgoingEdges[n] = source.outgoingEdges[n].slice(0);
        result.incomingEdges[n] = source.incomingEdges[n].slice(0);
      });
      return result;
    },
    /**
     * Get an array containing the nodes that the specified node depends on (transitively).
     *
     * Throws an Error if the graph has a cycle, or the specified node does not exist.
     *
     * If `leavesOnly` is true, only nodes that do not depend on any other nodes will be returned
     * in the array.
     */
    dependenciesOf:function (node, leavesOnly) {
      if (this.hasNode(node)) {
        var result = [];
        var DFS = createDFS(this.outgoingEdges, leavesOnly, result, this.circular);
        DFS(node);
        var idx = result.indexOf(node);
        if (idx >= 0) {
          result.splice(idx, 1);
        }
        return result;
      }
      else {
        throw new Error('Node does not exist: ' + node);
      }
    },
    /**
     * get an array containing the nodes that depend on the specified node (transitively).
     *
     * Throws an Error if the graph has a cycle, or the specified node does not exist.
     *
     * If `leavesOnly` is true, only nodes that do not have any dependants will be returned in the array.
     */
    dependantsOf:function (node, leavesOnly) {
      if (this.hasNode(node)) {
        var result = [];
        var DFS = createDFS(this.incomingEdges, leavesOnly, result, this.circular);
        DFS(node);
        var idx = result.indexOf(node);
        if (idx >= 0) {
          result.splice(idx, 1);
        }
        return result;
      } else {
        throw new Error('Node does not exist: ' + node);
      }
    },
    /**
     * Construct the overall processing order for the dependency graph.
     *
     * Throws an Error if the graph has a cycle.
     *
     * If `leavesOnly` is true, only nodes that do not depend on any other nodes will be returned.
     */
    overallOrder:function (leavesOnly) {
      var self = this;
      var result = [];
      var keys = Object.keys(this.nodes);
      if (keys.length === 0) {
        return result; // Empty graph
      } else {
        // Look for cycles - we run the DFS starting at all the nodes in case there
        // are several disconnected subgraphs inside this dependency graph.
        var CycleDFS = createDFS(this.outgoingEdges, false, [], this.circular);
        keys.forEach(function(n) {
          CycleDFS(n);
        });

        var DFS = createDFS(this.outgoingEdges, leavesOnly, result, this.circular);
        // Find all potential starting points (nodes with nothing depending on them) an
        // run a DFS starting at these points to get the order
        keys.filter(function (node) {
          return self.incomingEdges[node].length === 0;
        }).forEach(function (n) {
          DFS(n);
        });

        return result;
      }
    }
  };
  });
  var dep_graph_1 = dep_graph.DepGraph;

  var replace = function (str, find, repl) {
      // modified from http://jsperf.com/javascript-replace-all/10
      var orig = str.toString();
      var res = '';
      var rem = orig;
      var beg = 0;
      var end = rem.indexOf(find);
      while (end > -1) {
          res += orig.substring(beg, beg + end) + repl;
          rem = rem.substring(end + find.length, rem.length);
          beg += end + find.length;
          end = rem.indexOf(find);
      }
      if (rem.length > 0) {
          res += orig.substring(orig.length - rem.length, orig.length);
      }
      return res;
  };
  var decodeFragmentSegments = function (segments) {
      var len = segments.length;
      var res = new Array(len);
      var i = -1;
      while (++i < len) {
          res[i] = replace(replace(decodeURIComponent('' + segments[i]), '~1', '/'), '~0', '~');
      }
      return res;
  };
  var decodeUriFragmentIdentifier = function (ptr) {
      if (typeof ptr !== 'string') {
          throw new TypeError('Invalid type: JSON Pointers are represented as strings.');
      }
      if (ptr.length === 0 || ptr[0] !== '#') {
          throw new ReferenceError('Invalid JSON Pointer syntax; URI fragment idetifiers must begin with a hash.');
      }
      if (ptr.length === 1) {
          return [];
      }
      if (ptr[1] !== '/') {
          throw new ReferenceError('Invalid JSON Pointer syntax.');
      }
      return decodeFragmentSegments(ptr.substring(2).split('/'));
  };
  var encodeFragmentSegment = function (segment) {
      if (typeof segment === 'string') {
          return replace(replace(segment, '~', '~0'), '/', '~1');
      }
      return segment;
  };
  var jsonPointerToPath = function (pointer) {
      return decodeUriFragmentIdentifier(pointer);
  };
  var addToJSONPointer = function (pointer, part) {
      return pointer + "/" + encodeFragmentSegment(part);
  };

  var ResolveCrawler = /** @class */ (function () {
      function ResolveCrawler(runner, jsonPointer) {
          var _this = this;
          this.authorityResolvers = [];
          // @ts-ignore
          this.pointerGraph = new dep_graph_1({ circular: true });
          // @ts-ignore
          this.pointerStemGraph = new dep_graph_1({ circular: true });
          this.computeGraph = function (target, parentPath, parentPointer, pointerStack) {
              if (parentPath === void 0) { parentPath = []; }
              if (parentPointer === void 0) { parentPointer = '#'; }
              if (pointerStack === void 0) { pointerStack = []; }
              if (!parentPointer)
                  parentPointer = '#';
              var ref = _this._runner.computeRef({
                  val: target,
                  jsonPointer: parentPointer,
                  pointerStack: pointerStack,
              });
              /**
               * Is this entire target something that we want to resolve? For example:
               *
               * target = {
               *   "$ref": "#/hi"
               * }
               */
              if (ref) {
                  _this._resolveRef({
                      ref: ref,
                      val: target,
                      parentPath: parentPath,
                      pointerStack: pointerStack,
                      parentPointer: parentPointer,
                      cacheKey: parentPointer,
                      resolvingPointer: _this.jsonPointer,
                  });
              }
              else if (typeof target === 'object') {
                  for (var key in target) {
                      if (!target.hasOwnProperty(key))
                          continue;
                      var val = target[key];
                      var currentPointer = addToJSONPointer(parentPointer, key);
                      ref = _this._runner.computeRef({
                          key: key,
                          val: val,
                          jsonPointer: currentPointer,
                          pointerStack: pointerStack,
                      });
                      parentPath.push(key);
                      // if this value a ref, resolve and continue on to the next property
                      if (ref) {
                          _this._resolveRef({
                              ref: ref,
                              val: val,
                              parentPath: parentPath,
                              parentPointer: currentPointer,
                              pointerStack: pointerStack,
                              cacheKey: ref.toJSONPointer(),
                              resolvingPointer: _this.jsonPointer,
                          });
                      }
                      else if (typeof val === 'object') {
                          // recurse into the object
                          _this.computeGraph(val, parentPath, currentPointer, pointerStack);
                      }
                      parentPath.pop();
                  }
              }
          };
          this._resolveRef = function (opts) {
              var pointerStack = opts.pointerStack, parentPath = opts.parentPath, parentPointer = opts.parentPointer, ref = opts.ref;
              // local pointer
              if (ref.isJSONPointer()) {
                  if (_this._runner.resolvePointers) {
                      var targetPointer = ref.toJSONPointer();
                      var targetPath = jsonPointerToPath(targetPointer);
                      /**
                       * Protects against circular references back to something higher up in the tree
                       * Will stop #/definitions/columns/rows -> #/definitions/columns
                       */
                      var referencesParent = true;
                      for (var i in targetPath) {
                          if (parentPath[i] !== targetPath[i]) {
                              referencesParent = false;
                              break;
                          }
                      }
                      if (referencesParent)
                          return;
                      // pointerStemGraph tracks all of the stem dependency and is used later
                      // in the runner to protect against circular refs in the final JS object
                      if (!_this.pointerStemGraph.hasNode(targetPointer)) {
                          _this.pointerStemGraph.addNode(targetPointer);
                      }
                      var stem = '#';
                      var tail = '';
                      for (var i = 0; i < parentPath.length; i++) {
                          var part = parentPath[i];
                          if (part === targetPath[i]) {
                              stem += "/" + part;
                          }
                          else {
                              tail += "/" + part;
                              var dep = "" + stem + tail;
                              if (dep !== parentPointer && dep !== targetPointer) {
                                  if (!_this.pointerStemGraph.hasNode(dep)) {
                                      _this.pointerStemGraph.addNode(dep);
                                  }
                                  _this.pointerStemGraph.addDependency(dep, targetPointer);
                              }
                          }
                      }
                      if (!_this.pointerGraph.hasNode(parentPointer)) {
                          _this.pointerGraph.addNode(parentPointer);
                      }
                      if (!_this.pointerGraph.hasNode(targetPointer)) {
                          _this.pointerGraph.addNode(targetPointer);
                      }
                      // register parent as a dependant of the target
                      _this.pointerGraph.addDependency(parentPointer, targetPointer);
                      // if we are partially resolving, we need to follow refs (since they might point outside of our initial target object tree)
                      // only need to initiate when top of pointer stack
                      if (_this.jsonPointer && pointerStack.length === 0) {
                          pointerStack.push(targetPointer);
                          // if we are partially resolving
                          _this.computeGraph(json_4(_this._runner.source, targetPath), targetPath, targetPointer, pointerStack);
                          pointerStack.pop();
                      }
                  }
              }
              else {
                  // remote pointer
                  if (_this._runner.resolveAuthorities && !_this._runner.atMaxAuthorityDepth()) {
                      _this.authorityResolvers.push(_this._runner.lookupAndResolveAuthority(opts));
                  }
              }
          };
          this.jsonPointer = jsonPointer;
          this._runner = runner;
      }
      return ResolveCrawler;
  }());

  var memoize$2 = require('fast-memoize');
  var resolveRunnerCount = 0;
  var _removeRootPath = function (root, child) {
      var rootPathParts = root.split('/');
      var childPathParts = child.split('/');
      for (var _i = 0, rootPathParts_1 = rootPathParts; _i < rootPathParts_1.length; _i++) {
          var rootPathPart = rootPathParts_1[_i];
          if (!rootPathPart)
              continue;
          if (childPathParts.length === 0)
              break;
          if (rootPathPart === childPathParts[0]) {
              childPathParts.shift();
          }
          else {
              break;
          }
      }
      return childPathParts.join('/');
  };
  var _resolvePaths = function (root, child) {
      var filePath = root.split('/');
      filePath[filePath.length - 1] = child;
      return filePath.join('/');
  };
  var ResolveRunner = /** @class */ (function () {
      function ResolveRunner(source, opts) {
          if (opts === void 0) { opts = {}; }
          var _this = this;
          this.ctx = {};
          /**
           * Determine if we should resolve this part of source
           * If so, return the appropriate URI object
           */
          this.computeRef = function (opts) {
              var ref;
              if (opts.key === '$ref') {
                  ref = opts.val;
              }
              else if (opts.val && typeof opts.val === 'object' && opts.val.$ref) {
                  ref = opts.val.$ref;
              }
              if (!ref)
                  return;
              ref = uri_1.parse(ref);
              // Does ref only have a fragment
              if (ref.toString(true) !== "#" + ref.fragment) {
                  // if we're working with a file
                  if (ref.isFile(_this.authority)) {
                      if (_this.authority.scheme && !_this.authority.isFile()) {
                          // TODO: should error here, cannot resolve file refs if the parent is not a file (for example if parent was http)
                          return ref;
                      }
                      var resolvedPath = _resolvePaths(_this.authority.fsPath, ref.fsPath);
                      ref = ref.with({ scheme: 'file', path: resolvedPath });
                  }
                  else if ((ref.scheme.includes('http') ||
                      (ref.scheme === '' && _this.authority.scheme.includes('http'))) &&
                      ref.toString(true) !== _this.authority.toString(true)) {
                      if (_this.authority.authority !== '' && ref.authority === '') {
                          var child = _removeRootPath(_this.authority.fsPath, ref.fsPath);
                          var resolvedPath = _resolvePaths(_this.authority.fsPath, child);
                          ref = _this.authority.with({
                              path: resolvedPath,
                              fragment: ref.fragment,
                          });
                      }
                  }
              }
              if (_this.transformRef) {
                  return _this.transformRef(__assign({}, opts, { ref: ref, authority: _this.authority }), _this.ctx);
              }
              return ref;
          };
          this.atMaxAuthorityDepth = function () {
              return _this.authorityStack.length >= 100;
          };
          this.lookupAuthority = function (opts) { return __awaiter(_this, void 0, void 0, function () {
              var ref, reader, result;
              return __generator(this, function (_a) {
                  switch (_a.label) {
                      case 0:
                          ref = opts.ref;
                          reader = this.readers[ref.scheme];
                          if (!reader) {
                              throw new Error("No reader defined for scheme '" + ref.scheme + "' in ref " + ref.toString(true));
                          }
                          return [4 /*yield*/, reader.read(ref, this.ctx)];
                      case 1:
                          result = _a.sent();
                          return [2 /*return*/, new ResolveRunner(result, {
                                  depth: this.depth + 1,
                                  authority: ref,
                                  authorityStack: this.authorityStack,
                                  authorityCache: this.authorityCache,
                                  readers: this.readers,
                                  debug: this.debug,
                                  transformRef: this.transformRef,
                                  parseAuthorityResult: this.parseAuthorityResult,
                                  resolveAuthorities: this.resolveAuthorities,
                                  resolvePointers: this.resolvePointers,
                                  ctx: this.ctx,
                              })];
                  }
              });
          }); };
          this.lookupAndResolveAuthority = function (opts) { return __awaiter(_this, void 0, void 0, function () {
              var val, ref, resolvingPointer, parentPointer, pointerStack, parentPath, authorityCacheKey, lookupResult, authorityResolver, currentAuthority, e_1, _a, _i, _b, error, errorPathInResult, parsed, e_2, e_3;
              return __generator(this, function (_c) {
                  switch (_c.label) {
                      case 0:
                          val = opts.val, ref = opts.ref, resolvingPointer = opts.resolvingPointer, parentPointer = opts.parentPointer, pointerStack = opts.pointerStack;
                          parentPath = (opts.parentPath || []).slice();
                          authorityCacheKey = this.computeAuthorityCacheKey(ref);
                          lookupResult = {
                              pointerStack: pointerStack,
                              targetPath: resolvingPointer === parentPointer ? [] : parentPath,
                          };
                          if (!this.authorityStack.includes(authorityCacheKey)) return [3 /*break*/, 1];
                          lookupResult.resolved = {
                              result: val,
                              errors: [],
                              runner: this,
                          };
                          return [2 /*return*/, lookupResult];
                      case 1:
                          authorityResolver = void 0;
                          _c.label = 2;
                      case 2:
                          _c.trys.push([2, 4, , 5]);
                          if (this.atMaxAuthorityDepth()) {
                              // safe guard against edge cases we might not have caught yet..
                              // TODO: report this to bugsnag so we can track? throw it as some special
                              // fatal error, that platform can look for and report (maybe other errors as well)?
                              throw new Error("Max authority depth (" + this.authorityStack.length + ") reached. Halting, this is probably a circular loop.");
                          }
                          return [4 /*yield*/, this.lookupAuthority({
                                  ref: ref.with({ fragment: '' }),
                                  cacheKey: authorityCacheKey,
                              })];
                      case 3:
                          authorityResolver = _c.sent();
                          currentAuthority = this.authority.toString(true);
                          if (currentAuthority && this.depth !== 0) {
                              authorityResolver.authorityStack = authorityResolver.authorityStack.concat([
                                  currentAuthority,
                              ]);
                          }
                          return [3 /*break*/, 5];
                      case 4:
                          e_1 = _c.sent();
                          lookupResult.error = {
                              code: 'RESOLVE_AUTHORITY',
                              message: String(e_1),
                              authority: ref,
                              authorityStack: this.authorityStack,
                              pointerStack: pointerStack,
                              path: parentPath,
                          };
                          return [3 /*break*/, 5];
                      case 5:
                          if (!authorityResolver) return [3 /*break*/, 13];
                          _c.label = 6;
                      case 6:
                          _c.trys.push([6, 12, , 13]);
                          _a = lookupResult;
                          return [4 /*yield*/, authorityResolver.resolve(ref.toJSONPointer())];
                      case 7:
                          _a.resolved = _c.sent();
                          // if pointer resolution failed, revert to the original value (which will be a $ref most of the time)
                          if (lookupResult.resolved.errors.length) {
                              for (_i = 0, _b = lookupResult.resolved.errors; _i < _b.length; _i++) {
                                  error = _b[_i];
                                  if (error.code === 'POINTER_MISSING' &&
                                      error.path.join('/') === ref.fragment.slice(1) // only reset result value if the error is specifically for this fragment
                                  ) {
                                      errorPathInResult = ref.fragment
                                          ? json_7(error.path, json_7(ref.fragment, '/').split('/'))
                                          : error.path;
                                      if (errorPathInResult && errorPathInResult.length) {
                                          json_5(lookupResult.resolved.result, errorPathInResult, val);
                                      }
                                      else if (lookupResult.resolved.result) {
                                          lookupResult.resolved.result = val;
                                      }
                                  }
                              }
                          }
                          if (!this.parseAuthorityResult) return [3 /*break*/, 11];
                          _c.label = 8;
                      case 8:
                          _c.trys.push([8, 10, , 11]);
                          return [4 /*yield*/, this.parseAuthorityResult({
                                  authorityResult: lookupResult,
                                  result: lookupResult.resolved.result,
                                  targetAuthority: ref,
                                  parentAuthority: this.authority,
                                  parentPath: parentPath,
                              })];
                      case 9:
                          parsed = _c.sent();
                          // if (parsed.errors) {
                          // TODO: as mentioned above, allow caller to add errors/validations
                          // }
                          lookupResult.resolved.result = parsed.result;
                          return [3 /*break*/, 11];
                      case 10:
                          e_2 = _c.sent();
                          // could not parse... roll back to original value
                          lookupResult.resolved.result = val;
                          lookupResult.error = {
                              code: 'PARSE_AUTHORITY',
                              message: "Error parsing lookup result for '" + ref.toString(e_2) + "': " + String(e_2),
                              authority: ref,
                              authorityStack: this.authorityStack,
                              pointerStack: pointerStack,
                              path: parentPath,
                          };
                          return [3 /*break*/, 11];
                      case 11: return [3 /*break*/, 13];
                      case 12:
                          e_3 = _c.sent();
                          lookupResult.error = {
                              code: 'RESOLVE_POINTER',
                              message: "Error resolving pointer @ " + ref.toJSONPointer() + ": " + String(e_3),
                              path: parentPath,
                              authority: ref,
                              authorityStack: this.authorityStack,
                              pointerStack: pointerStack,
                          };
                          return [3 /*break*/, 13];
                      case 13: return [2 /*return*/, lookupResult];
                  }
              });
          }); };
          this.id = resolveRunnerCount += 1;
          this.depth = opts.depth || 0;
          this._source = source;
          this.authority = opts.authority || uri_1.parse('');
          this.authorityStack = opts.authorityStack || [];
          this.authorityCache = opts.authorityCache || new Cache();
          if (this.authority && this.depth === 0) {
              // if this first runner is an authority, seed the cache so we don't create another one for
              // this authority later
              this.authorityCache.set(this.computeAuthorityCacheKey(this.authority), this);
          }
          this.readers = opts.readers || {};
          this.parseAuthorityResult = opts.parseAuthorityResult;
          this.transformRef = opts.transformRef;
          this.debug = opts.debug || false;
          this.resolvePointers =
              typeof opts.resolvePointers !== 'undefined' ? opts.resolvePointers : true;
          this.resolveAuthorities =
              typeof opts.resolveAuthorities !== 'undefined' ? opts.resolveAuthorities : true;
          this.ctx = opts.ctx;
          this.lookupAuthority = memoize$2(this.lookupAuthority, {
              serializer: this._cacheKeySerializer,
              cache: {
                  create: function () {
                      return _this.authorityCache;
                  },
              },
          });
      }
      Object.defineProperty(ResolveRunner.prototype, "source", {
          get: function () {
              return this._source;
          },
          enumerable: true,
          configurable: true
      });
      ResolveRunner.prototype.resolve = function (jsonPointer) {
          return __awaiter(this, void 0, void 0, function () {
              var resolved, targetPath, crawler, authorityResults, _loop_1, this_1, _i, authorityResults_1, r;
              var _this = this;
              return __generator(this, function (_a) {
                  switch (_a.label) {
                      case 0:
                          resolved = {
                              result: this.source,
                              errors: [],
                              runner: this,
                          };
                          jsonPointer = jsonPointer && jsonPointer.trim();
                          if (jsonPointer && jsonPointer !== '#' && jsonPointer !== '#/') {
                              targetPath = jsonPointerToPath(jsonPointer);
                              resolved.result = json_4(resolved.result, targetPath);
                          }
                          if (!resolved.result) {
                              resolved.errors.push({
                                  code: 'POINTER_MISSING',
                                  message: "'" + jsonPointer + "' does not exist @ '" + this.authority.toString(true) + "'",
                                  authority: this.authority,
                                  authorityStack: this.authorityStack,
                                  pointerStack: [],
                                  path: targetPath || [],
                              });
                              return [2 /*return*/, resolved];
                          }
                          crawler = new ResolveCrawler(this, jsonPointer);
                          // crawl to build up the authorityResolvers and pointerGraph
                          crawler.computeGraph(resolved.result, targetPath, jsonPointer || '');
                          authorityResults = [];
                          if (!crawler.authorityResolvers.length) return [3 /*break*/, 2];
                          return [4 /*yield*/, Promise.all(crawler.authorityResolvers)];
                      case 1:
                          authorityResults = _a.sent();
                          _a.label = 2;
                      case 2:
                          // wrap all the mutations in a producer, for structural sharing + immutability
                          // Wait for all of the authority resolvers to complete
                          if (authorityResults.length) {
                              _loop_1 = function (r) {
                                  if (r.error) {
                                      resolved.errors.push(r.error);
                                  }
                                  if (!r.resolved)
                                      return "continue";
                                  if (r.resolved.errors) {
                                      resolved.errors = resolved.errors.concat(r.resolved.errors);
                                  }
                                  if (!r.resolved.result)
                                      return "continue";
                                  // does this resolved result belong somewhere specific in the source data?
                                  var resolvedTargetPath = r.targetPath;
                                  // if not, we should set on our targetPath
                                  if (!resolvedTargetPath.length)
                                      resolvedTargetPath = targetPath || [];
                                  this_1._source = produce(this_1._source, function (draft) {
                                      if (r.resolved) {
                                          if (!resolvedTargetPath.length) {
                                              return r.resolved.result;
                                          }
                                          else {
                                              json_5(draft, resolvedTargetPath, r.resolved.result);
                                          }
                                      }
                                  });
                              };
                              this_1 = this;
                              // Set the authority resolver results correctly
                              for (_i = 0, authorityResults_1 = authorityResults; _i < authorityResults_1.length; _i++) {
                                  r = authorityResults_1[_i];
                                  _loop_1(r);
                              }
                          }
                          if (typeof this._source !== 'object') {
                              resolved.result = this._source;
                              return [2 /*return*/, resolved];
                          }
                          // If using parseAuthorityResult, do not need to replace local pointers here (parseAuthorityResult is responsible)
                          // if this is not an authority, then we should parse even if parseAuthorityResult is present
                          if (this.resolvePointers) {
                              this._source = produce(this._source, function (draft) {
                                  var processOrder = [];
                                  try {
                                      processOrder = crawler.pointerGraph.overallOrder();
                                      // loop through the pointer graph in the correct order, setting values we go
                                      // this is where local pointers are replaced with their resolved values
                                      for (var _i = 0, processOrder_1 = processOrder; _i < processOrder_1.length; _i++) {
                                          var pointer = processOrder_1[_i];
                                          var dependants = crawler.pointerGraph.dependantsOf(pointer);
                                          if (!dependants.length)
                                              continue;
                                          var pointerPath = jsonPointerToPath(pointer);
                                          var val = json_4(draft, pointerPath);
                                          for (var _a = 0, dependants_1 = dependants; _a < dependants_1.length; _a++) {
                                              var dependant = dependants_1[_a];
                                              // check to prevent circular references in the resulting JS object
                                              // this implementation is MUCH more performant than decycling the final object to remove circulars
                                              var isCircular = void 0;
                                              var dependantPath = jsonPointerToPath(dependant);
                                              var dependantStems = crawler.pointerStemGraph.dependenciesOf(pointer);
                                              for (var _b = 0, dependantStems_1 = dependantStems; _b < dependantStems_1.length; _b++) {
                                                  var stem = dependantStems_1[_b];
                                                  if (json_6(dependantPath, jsonPointerToPath(stem))) {
                                                      isCircular = true;
                                                      break;
                                                  }
                                              }
                                              // TODO: we might want to track and expose these circulars in the future?
                                              if (isCircular)
                                                  continue;
                                              if (val) {
                                                  json_5(draft, dependantPath, val);
                                              }
                                              else {
                                                  resolved.errors.push({
                                                      code: 'POINTER_MISSING',
                                                      message: "'" + pointer + "' does not exist",
                                                      path: dependantPath,
                                                      authority: _this.authority,
                                                      authorityStack: _this.authorityStack,
                                                      pointerStack: [],
                                                  });
                                              }
                                          }
                                      }
                                  }
                                  catch (e) {
                                      // (MM) TODO: report this error? usually means some sort of uncaught circular structure
                                  }
                              });
                          }
                          if (targetPath) {
                              resolved.result = json_4(this._source, targetPath);
                          }
                          else {
                              resolved.result = this._source;
                          }
                          return [2 /*return*/, resolved];
                  }
              });
          });
      };
      ResolveRunner.prototype._cacheKeySerializer = function (sOpts) {
          return sOpts && typeof sOpts === 'object' && sOpts.cacheKey
              ? sOpts.cacheKey
              : JSON.stringify(arguments);
      };
      ResolveRunner.prototype.computeAuthorityCacheKey = function (ref) {
          // don't include the fragment on authority cache key
          return ref.with({ fragment: '' }).toString(true);
      };
      return ResolveRunner;
  }());

  var Resolver = /** @class */ (function () {
      function Resolver(opts) {
          if (opts === void 0) { opts = {}; }
          this.ctx = {};
          this.authorityCache = opts.authorityCache || new Cache(opts.authorityCacheOpts);
          this.readers = opts.readers || {};
          this.debug = opts.debug || false;
          this.transformRef = opts.transformRef;
          this.resolvePointers = opts.resolvePointers || true;
          this.resolveAuthorities = opts.resolveAuthorities || true;
          this.parseAuthorityResult = opts.parseAuthorityResult;
          this.ctx = opts.ctx;
      }
      Resolver.prototype.resolve = function (source, opts) {
          if (opts === void 0) { opts = {}; }
          var runOpts = Object.assign({}, {
              authorityCache: this.authorityCache,
              readers: this.readers,
              debug: this.debug,
              transformRef: this.transformRef,
              resolvePointers: this.resolvePointers,
              resolveAuthorities: this.resolveAuthorities,
              parseAuthorityResult: this.parseAuthorityResult,
          }, opts);
          // console.log(JSON.stringify(runOpts, null, 2));
          // merge ctx
          runOpts.ctx = Object.assign({}, this.ctx || {}, opts.ctx || {});
          var runner = new ResolveRunner(source, runOpts);
          return runner.resolve(opts.jsonPointer);
      };
      return Resolver;
  }());

  exports.Resolver = Resolver;

  Object.defineProperty(exports, '__esModule', { value: true });

})));
//# sourceMappingURL=json-ref-resolver.umd.js.map
